package cn.ezhizao.project.business.carrierReject.controller;

import java.util.List;
import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;

import cn.ezhizao.common.utils.poi.ExcelUtil;
import cn.ezhizao.framework.aspectj.lang.annotation.Log;
import cn.ezhizao.framework.aspectj.lang.enums.BusinessType;
import cn.ezhizao.framework.web.controller.BaseController;
import cn.ezhizao.framework.web.domain.AjaxResult;
import cn.ezhizao.framework.web.page.TableDataInfo;
import cn.ezhizao.project.business.carrierReject.domain.BizCarrierReject;
import cn.ezhizao.project.business.carrierReject.service.IBizCarrierRejectService;
import cn.ezhizao.project.business.product.domain.BizCarrier;
import cn.ezhizao.project.business.product.service.IBizCarrierService;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * 企业租户Controller
 *
 * @author ezhizao
 * @date 2024-01-09
 */
@RestController
@RequestMapping("/business/carrierReject")
public class BizCarrierRejectController extends BaseController
{
    @Resource
    private IBizCarrierRejectService bizCarrierRejectService;

    @Resource
    private IBizCarrierService carrierService;

    /**
     * 查询企业租户列表
     */
//    @PreAuthorize("@ss.hasPermi('business:carrierReject:list')")
    @GetMapping("/list")
    public TableDataInfo list(BizCarrierReject bizCarrierReject) throws NoSuchFieldException, IllegalAccessException
    {
        setTenantId(bizCarrierReject);
        startPage();
        List<BizCarrierReject> list = bizCarrierRejectService.getList(bizCarrierReject);
        return getDataTable(list);
    }

    /**
     * 导出企业租户列表
     */
//    @PreAuthorize("@ss.hasPermi('business:carrierReject:export')")
    @Log(title = "企业租户", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, BizCarrierReject bizCarrierReject) throws NoSuchFieldException, IllegalAccessException
    {
        setTenantId(bizCarrierReject);
        List<BizCarrierReject> list = bizCarrierRejectService.getList(bizCarrierReject);
        ExcelUtil<BizCarrierReject> util = new ExcelUtil<BizCarrierReject>(BizCarrierReject.class);
        util.exportExcel(response, list, "企业租户数据");
    }

    /**
     * 获取企业租户详细信息
     */
//    @PreAuthorize("@ss.hasPermi('business:carrierReject:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(bizCarrierRejectService.getById(id));
    }

    /**
     * 新增企业租户
     */
//    @PreAuthorize("@ss.hasPermi('business:carrierReject:add')")
    @Log(title = "企业租户", businessType = BusinessType.INSERT)
    @PostMapping(value = "/save")
    public AjaxResult add(@RequestBody List<BizCarrierReject> bizCarrierReject) throws NoSuchFieldException, IllegalAccessException
    {
        for (BizCarrierReject item: bizCarrierReject
             ) {
            BizCarrier carrier = new BizCarrier();
            carrier.setId(item.getCarrierId());
            BizCarrier bizCarrier = carrierService.getList(carrier).get(0);
            bizCarrier.setIsAbandoned(1);
            carrierService.updateById(bizCarrier);
        }
        return toAjax(bizCarrierRejectService.saveBatch(bizCarrierReject));
    }

    /**
     * 修改企业租户
     */
//    @PreAuthorize("@ss.hasPermi('business:reject:edit')")
    @Log(title = "企业租户", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody BizCarrierReject bizCarrierReject) throws NoSuchFieldException, IllegalAccessException
    {
        setTenantId(bizCarrierReject);
        return toAjax(bizCarrierRejectService.updateById(bizCarrierReject));
    }

    /**
     * 删除企业租户
     */
//    @PreAuthorize("@ss.hasPermi('business:reject:remove')")
    @Log(title = "企业租户", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable List<Long> ids)
    {
        return toAjax(bizCarrierRejectService.removeBatchByIds(ids));
    }
}
