package cn.ezhizao.project.business.deptProcess.controller;

import cn.ezhizao.common.utils.poi.ExcelUtil;
import cn.ezhizao.framework.aspectj.lang.annotation.Log;
import cn.ezhizao.framework.aspectj.lang.enums.BusinessType;
import cn.ezhizao.framework.web.controller.BaseController;
import cn.ezhizao.framework.web.domain.AjaxResult;
import cn.ezhizao.project.business.deptProcess.domain.BizDeptProcess;
import cn.ezhizao.project.business.deptProcess.service.IBizDeptProcessService;
import cn.ezhizao.project.business.process.domain.BizProcess;
import cn.ezhizao.project.business.product.domain.BizDayworkItem;
import cn.ezhizao.project.business.product.domain.BizProductionPlanDetail;
import cn.ezhizao.project.business.product.service.IBizDayworkItemService;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.*;
import java.util.stream.Collectors;

import cn.ezhizao.project.business.product.service.IBizProductionPlanDetailService;
import cn.ezhizao.project.business.technologicalProcessDetail.domain.BizTechnologicalProcessDetail;
import cn.ezhizao.project.business.technologicalProcessDetail.service.IBizTechnologicalProcessDetailService;
import org.springframework.web.bind.annotation.*;


/**
 * deptProcessController
 *
 * @author ezhizao
 * @date 2023-12-11
 */
@RestController
@RequestMapping("/business/deptProcess")
public class BizDeptProcessController extends BaseController {
    @Resource
    private IBizDeptProcessService bizDeptProcessService;

    @Resource
    private IBizDayworkItemService bizDayworkItemService;

    @Resource
    private IBizTechnologicalProcessDetailService bizTechnologicalProcessDetailService;

    @Resource
    private IBizProductionPlanDetailService bizProductionPlanDetailService;

    @Resource
    HttpServletRequest request;

    /**
     * 查询deptProcess列表
     */
//    @PreAuthorize("@ss.hasPermi('business:deptProcess:list')")
    @PostMapping("/list")
    public AjaxResult list(@RequestBody Map<String, Object> data) throws NoSuchFieldException, IllegalAccessException {
        List<String> statusList = Arrays.asList("0", "1", "2");
        Long deptId = Long.parseLong(data.get("deptId").toString());
        List<BizDeptProcess> deptProcesses = bizDeptProcessService.query().eq("dept_id", deptId).list();
        List<Long> deptProcessIds = deptProcesses.stream().map(BizDeptProcess::getProcessId).collect(Collectors.toList());

        long dayworkId = 0L;
        if (data.get("dayworkId") != null) {
            dayworkId = Long.parseLong(data.get("dayworkId").toString());
        }

        long planDetailId = 0L;
        if (data.get("planDetailId") != null) {
            planDetailId = Long.parseLong(data.get("planDetailId").toString());
            BizProductionPlanDetail productionPlanDetail = bizProductionPlanDetailService.getById(planDetailId);
            List<BizTechnologicalProcessDetail> technologicalProcessDetails = bizTechnologicalProcessDetailService.query()
                    .eq("technological_process_id", productionPlanDetail.getTechnologicalProcessId())
                    .ne("tenant_id", 9)
                    .in("process_id", deptProcessIds)
                    .orderByAsc("process_step_number")
                    .list();

            return success(technologicalProcessDetails);
        }
        /* 当一个工序未工序完成，再开始报工时，只能做当前工序，不允许做其它工序
        if (dayworkId > 0){
            List<BizDayworkItem> itemList = bizDayworkItemService.query()
                    .eq("daywork_id", dayworkId)
                    .in("status", statusList)
                    .orderByDesc("create_time")
                    .list();
            //如果当前批次有未完成的工序，那么只能做这一道工序
            for (BizDeptProcess item : list) {
                if (!itemList.isEmpty()){
                    //保留工段下和他相同的那一条工序
                    if(itemList.get(0).getProcessId().equals(item.getProcessId())){
                        returnList.add(item);
                        return success(returnList);
                    }
                }
            }
        }
        */
        return success(deptProcesses);
    }


    @GetMapping("/getSpecialDeptProcessList")
    public AjaxResult getSpecialDeptProcessList() throws NoSuchFieldException, IllegalAccessException {
        List<BizDeptProcess> list = bizDeptProcessService.getSpecialDeptProcessList();
        return success(list);
    }

    /**
     * 导出deptProcess列表
     */
//    @PreAuthorize("@ss.hasPermi('business:deptProcess:export')")
    @Log(title = "deptProcess", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, BizDeptProcess bizDeptProcess) throws NoSuchFieldException, IllegalAccessException {
        setTenantId(bizDeptProcess);
        List<BizDeptProcess> list = bizDeptProcessService.getList(bizDeptProcess);
        ExcelUtil<BizDeptProcess> util = new ExcelUtil<BizDeptProcess>(BizDeptProcess.class);
        util.exportExcel(response, list, "deptProcess数据");
    }

    /**
     * 获取deptProcess详细信息
     */
//    @PreAuthorize("@ss.hasPermi('business:deptProcess:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id) {
        return success(bizDeptProcessService.getById(id));
    }

    /**
     * 新增deptProcess
     */
//    @PreAuthorize("@ss.hasPermi('business:deptProcess:add')")
    @Log(title = "deptProcess", businessType = BusinessType.INSERT)
    @PostMapping("/save")
    public AjaxResult add(@RequestBody List<BizDeptProcess> bizDeptProcess) throws NoSuchFieldException, IllegalAccessException {
        final String tenantId = request.getHeader("tenantId");
        for (BizDeptProcess deptProcess : bizDeptProcess
        ) {
            deptProcess.setTenantId(Long.valueOf(tenantId));
        }
        List<BizDeptProcess> oldList = bizDeptProcessService.query().eq("dept_id", bizDeptProcess.get(0).getDeptId()).eq("tenant_id", bizDeptProcess.get(0).getTenantId()).list();
        boolean batch = saveOrUpdateBatch(bizDeptProcessService, bizDeptProcess, oldList);
        return toAjax(batch);
    }

    /**
     * 修改deptProcess
     */
//    @PreAuthorize("@ss.hasPermi('business:deptProcess:edit')")
    @Log(title = "deptProcess", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody BizDeptProcess bizDeptProcess) throws NoSuchFieldException, IllegalAccessException {
        final String tenantId = request.getHeader("tenantId");
        bizDeptProcess.setTenantId(Long.valueOf(tenantId));
        return toAjax(bizDeptProcessService.updateById(bizDeptProcess));
    }

    /**
     * 删除deptProcess
     */
//    @PreAuthorize("@ss.hasPermi('business:deptProcess:remove')")
    @Log(title = "deptProcess", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable List<Long> ids) {
        return toAjax(bizDeptProcessService.removeBatchByIds(ids));
    }
}
