package cn.ezhizao.project.business.outsource.controller;

import java.util.List;
import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;

import cn.ezhizao.common.utils.poi.ExcelUtil;
import cn.ezhizao.framework.aspectj.lang.annotation.Log;
import cn.ezhizao.framework.aspectj.lang.enums.BusinessType;
import cn.ezhizao.framework.web.controller.BaseController;
import cn.ezhizao.framework.web.domain.AjaxResult;
import cn.ezhizao.framework.web.page.TableDataInfo;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import cn.ezhizao.project.business.outsource.domain.BizOutsourcedOrderDetailProcess;
import cn.ezhizao.project.business.outsource.service.IBizOutsourcedOrderDetailProcessService;

/**
 * 外协单从对应的外协工序的从Controller
 *
 * @author ezhizao
 * @date 2024-05-28
 */
@RestController
@RequestMapping("/business/outsourcedOrderDetailProcess")
public class BizOutsourcedOrderDetailProcessController extends BaseController
{
    @Resource
    private IBizOutsourcedOrderDetailProcessService bizOutsourcedOrderDetailProcessService;

    /**
     * 查询外协单从对应的外协工序的从列表
     */
    @GetMapping("/list")
    public TableDataInfo list(BizOutsourcedOrderDetailProcess bizOutsourcedOrderDetailProcess) throws NoSuchFieldException, IllegalAccessException
    {
        setTenantId(bizOutsourcedOrderDetailProcess);
        startPage();
        List<BizOutsourcedOrderDetailProcess> list = bizOutsourcedOrderDetailProcessService.getList(bizOutsourcedOrderDetailProcess);
        return getDataTable(list);
    }

    /**
     * 导出外协单从对应的外协工序的从列表
     */
    @Log(title = "外协单从对应的外协工序的从", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, BizOutsourcedOrderDetailProcess bizOutsourcedOrderDetailProcess) throws NoSuchFieldException, IllegalAccessException
    {
        setTenantId(bizOutsourcedOrderDetailProcess);
        List<BizOutsourcedOrderDetailProcess> list = bizOutsourcedOrderDetailProcessService.getList(bizOutsourcedOrderDetailProcess);
        ExcelUtil<BizOutsourcedOrderDetailProcess> util = new ExcelUtil<BizOutsourcedOrderDetailProcess>(BizOutsourcedOrderDetailProcess.class);
        util.exportExcel(response, list, "外协单从对应的外协工序的从数据");
    }

    /**
     * 获取外协单从对应的外协工序的从详细信息
     */
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(bizOutsourcedOrderDetailProcessService.getById(id));
    }

    /**
     * 新增外协单从对应的外协工序的从
     */
    @Log(title = "外协单从对应的外协工序的从", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody BizOutsourcedOrderDetailProcess bizOutsourcedOrderDetailProcess) throws NoSuchFieldException, IllegalAccessException
    {
        setTenantId(bizOutsourcedOrderDetailProcess);
        return toAjax(bizOutsourcedOrderDetailProcessService.save(bizOutsourcedOrderDetailProcess));
    }

    /**
     * 修改外协单从对应的外协工序的从
     */
    @Log(title = "外协单从对应的外协工序的从", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody BizOutsourcedOrderDetailProcess bizOutsourcedOrderDetailProcess) throws NoSuchFieldException, IllegalAccessException
    {
        setTenantId(bizOutsourcedOrderDetailProcess);
        return toAjax(bizOutsourcedOrderDetailProcessService.updateById(bizOutsourcedOrderDetailProcess));
    }

    /**
     * 删除外协单从对应的外协工序的从
     */
    @Log(title = "外协单从对应的外协工序的从", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable List<Long> ids)
    {
        return toAjax(bizOutsourcedOrderDetailProcessService.removeBatchByIds(ids));
    }
}
