package cn.ezhizao.project.business.product.controller;

import java.util.*;
import java.util.stream.Collectors;
import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;

import cn.ezhizao.common.utils.poi.ExcelUtil;
import cn.ezhizao.framework.aspectj.lang.annotation.Log;
import cn.ezhizao.framework.aspectj.lang.enums.BusinessType;
import cn.ezhizao.framework.web.controller.BaseController;
import cn.ezhizao.framework.web.domain.AjaxResult;
import cn.ezhizao.framework.web.page.TableDataInfo;
import cn.ezhizao.project.business.carrierReject.domain.BizCarrierReject;
import cn.ezhizao.project.business.carrierReject.service.IBizCarrierRejectService;
import cn.ezhizao.project.business.product.domain.*;
import cn.ezhizao.project.business.product.service.*;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.aspectj.weaver.loadtime.Aj;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * 载具详情Controller
 *
 * @author ezhizao
 * @date 2023-12-04
 */
@RestController
@RequestMapping("/business/carrier")
public class BizCarrierController extends BaseController {
    @Resource
    private IBizCarrierService bizCarrierService;

    @Resource
    private IBizDayworkCarrierService bizDayworkCarrierService;
    @Resource
    private IBizCarrierCategoryService bizCarrierCategoryService;

    @Resource
    private IBizProcessInspecionService bizProcessInspecionService;
    @Resource
    private IBizDayworkService bizDayworkService;
    @Resource
    private IBizDayworkItemService bizDayworkItemService;

    @Resource
    private IBizCarrierRejectService bizCarrierRejectService;
    /**
     * 查询载具详情列表
     */
//    @PreAuthorize("@ss.hasPermi('business:carrier:list')")
    @GetMapping("/list")
    public TableDataInfo list(BizCarrier bizCarrier) throws NoSuchFieldException, IllegalAccessException {

        List<BizCarrier> list = bizCarrierService.getList(bizCarrier);
        return getDataTable(list);
    }

    /**
     * 导出载具详情列表
     */
//    @PreAuthorize("@ss.hasPermi('business:carrier:export')")
    @Log(title = "载具详情", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, BizCarrier bizCarrier) throws NoSuchFieldException, IllegalAccessException {
        List<BizCarrier> list = bizCarrierService.getList(bizCarrier);
        ExcelUtil<BizCarrier> util = new ExcelUtil<BizCarrier>(BizCarrier.class);
        util.exportExcel(response, list, "载具详情数据");
    }

    /**
     * 获取载具详情详细信息
     */
//    @PreAuthorize("@ss.hasPermi('business:carrier:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable Long id) {
        BizCarrier curCarrier = bizCarrierService.getOne(new QueryWrapper<BizCarrier>().eq("id", id));
        BizCarrierCategory carrierCategory = bizCarrierService.getCarrierCategory(curCarrier.getCategoryId());
        List<BizDayworkCarrier> dayworkCarrierList = bizDayworkCarrierService.query().eq("is_changed", 0).eq("carrier_id", id).list();
//        if (dayworkCarrierList.size() > 0 && curCarrier.getIsAllowMore() == 0) { // 单独通过箱子的字段判断
//            return error("该载具已被使用，未解绑无法使用");
//        }
        if (dayworkCarrierList.size() > 0 && carrierCategory.getIsAllowMore() == 0) { // 通过类别判断
            return error("该载具已被使用，未解绑无法使用");
        }
        BizCarrier carrier = new BizCarrier();
        carrier.setId(id);
        BizCarrier bizCarrier = bizCarrierService.getList(carrier).get(0);
        if (bizCarrier.getIsAbandoned() == 1) {
            return error("该载具已被废弃");
        }

        return success(bizCarrier);
    }

    /**
     * 新增载具详情
     */
//    @PreAuthorize("@ss.hasPermi('business:carrier:add')")
    @Log(title = "载具详情", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody BizCarrier bizCarrier) throws NoSuchFieldException, IllegalAccessException {
        return toAjax(bizCarrierService.save(bizCarrier));
    }

    /**
     * 修改载具详情
     */
//    @PreAuthorize("@ss.hasPermi('business:carrier:edit')")
    @Log(title = "载具详情", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody BizCarrier bizCarrier) throws NoSuchFieldException, IllegalAccessException {
        return toAjax(bizCarrierService.updateById(bizCarrier));
    }

    /**
     * 删除载具详情
     */
//    @PreAuthorize("@ss.hasPermi('business:carrier:remove')")
    @Log(title = "载具详情", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable List<Long> ids) {
        return toAjax(bizCarrierService.removeBatchByIds(ids));
    }

    /**
     * 判断一箱多批状态是否有未完成
     */

    @Log(title = "一箱多批状态", businessType = BusinessType.SELECT)
    @PostMapping("/carrierFinishedCheck")
    public AjaxResult carrierFinishedCheck(@RequestBody BizDaywork bizDaywork) {
        if(bizDaywork.getId() == null || bizDaywork.getId().equals(0L)) {
            return AjaxResult.error("请确认周转报工信息正确。");
        }

        // 获取该报工对应箱。
        List<BizDayworkCarrier> dayworkCarriers = bizDayworkCarrierService.query().eq("is_changed", 0).eq("daywork_id", bizDaywork.getId()).list();
        List<BizDayworkItem> currentItem = bizDayworkItemService.query().eq("daywork_id", bizDaywork.getId()).orderByDesc("create_time").list();
        List<BizCarrier> carriers = bizCarrierService.query().in("id", dayworkCarriers.isEmpty() ? Collections.singletonList(0L) : dayworkCarriers.stream().map(BizDayworkCarrier::getCarrierId).collect(Collectors.toList())).list();
        List<BizCarrierReject> carrierRejects = bizCarrierRejectService.query().in("carrier_id", carriers.isEmpty() ? Collections.singletonList(0L) : carriers.stream().map(BizCarrier::getId).collect(Collectors.toList())).list();
        carriers.stream().forEach(v->{v.setIsAbandoned(0);});
        if (carriers.isEmpty()) {
            // 该报工没有周转箱 提示选择周转箱
            return AjaxResult.error("请选择周转箱");
        }
        if(!carrierRejects.isEmpty()) {
            for (BizCarrier carrier : carriers) {
                BizCarrierReject bizCarrierReject = carrierRejects.stream().filter(v -> v.getCarrierId().equals(v.getCarrierId())).sorted(Comparator.comparing(BizCarrierReject::getCreateTime).reversed()).collect(Collectors.toList()).get(0);
                if (bizCarrierReject.getCarrierId().equals(carrier.getId())) {
                    carrier.setIsAbandoned(bizCarrierReject.getIsAbandoned());
                }
            }
        }
        if (carriers.stream().anyMatch(v -> v.getIsAbandoned().equals(1))) {
            return AjaxResult.error(carriers.stream().filter(v -> v.getIsAbandoned().equals(1)).map(BizCarrier::getCarrierCode).collect(Collectors.joining(",")) + "周转箱已废弃");
        }
        // 判断箱子类型，假设既有多批的箱子又有非多批的箱子，报错。
        List<BizCarrierCategory> bizCarrierCategories = bizCarrierCategoryService.query().in( "id", carriers.isEmpty() ? Collections.singletonList(0L): carriers.stream().map(BizCarrier::getCategoryId).collect(Collectors.toList())).list();
        if (bizCarrierCategories.stream().anyMatch(v -> v.getIsAllowMore().equals(1)) && bizCarrierCategories.stream().anyMatch(v -> v.getIsAllowMore().equals(0))) {
            return AjaxResult.error("周转箱既有铁箱又有蓝箱，请选择正确载具。");
        }

        // 判断箱子类型，假设是可以装多批，且箱子有多个 报错
        if (bizCarrierCategories.stream().allMatch(v -> v.getIsAllowMore().equals(1)) && carriers.size() > 1) {
            return AjaxResult.error("该批次关联多个铁箱，请选择正确的载具。");
        }
        // 获取这些箱对应的所有批次
        List<BizDayworkCarrier> carrierDaywork = bizDayworkCarrierService.query().eq("is_changed", 0).in("carrier_id",carriers.isEmpty() ? Collections.singletonList(0L) : carriers.stream().map(BizCarrier::getId).collect(Collectors.toList())).list();
        // 获取所有批次的报工状态
        List<BizDaywork> bizDayworks = bizDayworkService.query().in("id",carrierDaywork.isEmpty() ? Collections.singletonList(0L) : carrierDaywork.stream().map(BizDayworkCarrier::getDayworkId).collect(Collectors.toList())).list();
        List<BizDayworkItem> bizDayworkItems = bizDayworkItemService.query().in("daywork_id",bizDayworks.isEmpty() ? Collections.singletonList(0L) : bizDayworks.stream().map(BizDaywork::getId).collect(Collectors.toList())).list();
        // 判断箱子类型，假设是不可装多批则只能有一个批次，假设有多个 报错
        if (bizCarrierCategories.stream().allMatch(v -> v.getIsAllowMore().equals(0)) && bizDayworks.size() > 1) {
            return AjaxResult.error("存在蓝箱关联多个批次，请选择正确箱号");
        }
        for (BizDaywork daywork : bizDayworks) {
            List<BizDayworkItem> item = bizDayworkItems.stream().filter(v -> v.getDayworkId().equals(daywork.getId()) && v.getTechnologicalProcessDetailId().equals(currentItem.get(0).getTechnologicalProcessDetailId())).sorted(Comparator.comparing(BizDayworkItem::getStatus).reversed()).collect(Collectors.toList());
            // 判断是同一工序且完成可以周转
            // 有未完成则不能周转
            // 假设该批次进行了单批单改 这里可能有变化
            if (item.size() == 0 || Integer.parseInt(item.get(0).getStatus()) < 3 || !Objects.equals(item.get(0).getTechnologicalProcessDetailId(), currentItem.get(0).getTechnologicalProcessDetailId())) {
                return AjaxResult.error("该周转箱中批次未全部完成。");
            }
        }

        //是否是一箱多批
        if(bizDayworks.size() == 1){
            //查询该批批次的序捡状态
            BizProcessInspecion processInspecion =new BizProcessInspecion();
            processInspecion.setCarrierId(bizDayworks.get(0).getId());

            //根据报工信息查询说否有已完成的序检
            List<BizProcessInspecion> processInspecions = bizProcessInspecionService.query().eq("daywork_id",bizDayworks.get(0).getId()).orderByDesc("create_time").list();
            boolean perform = true;

            //如果存在判断是是否有当前序的完成
            if(processInspecions.size()>0) {
                BizProcessInspecion inspection = processInspecions.get(0);
                //如果存在判断是是否有当前序的完成
                if (inspection.getStatus() != 1) {
                    if (inspection.getStatus() == 0) {
                        return AjaxResult.error("该批次序检未完成，不能周转。");
                    }
                    if (inspection.getStatus() == 2) {
                        return AjaxResult.error("该批次序检不合格，不能周转。");
                    }
                    return AjaxResult.error("该批次序检未合格，不能周转。");
                }
            }
//            //如果存在判断是是否有当前序的完成
//            if(processInspecions.size()>0){
//                boolean isExist = bizDayworkItems.stream()
//                        .filter(v -> v.getIsProcessInspection().equals(1))
//                        .anyMatch(v -> v.getId().equals(processInspecion.getDayworkItemId()));
//                if(!isExist){
//                    return AjaxResult.error("该批次序检未合格，不能周转。");
//                }
//            }else{
//                return AjaxResult.error("该批次序检未合格，不能周转。");
//            }
        }else{
            if(!selectProcessInspecion(bizDayworks,bizDayworkItems)){
                return AjaxResult.error("该周转箱中批次序检未全部合格，不能周转。");
            }

        }


        // 全部完成可以周转
        return AjaxResult.success();
    }

    //查询一箱多批是否可以周转
    private boolean selectProcessInspecion (List<BizDaywork> bizDayworks,List<BizDayworkItem> bizDayworkItems){
        boolean perform = true;
        for (BizDaywork bizDaywork : bizDayworks) {
            //查询该批批次的序捡状态
            BizProcessInspecion processInspecion =new BizProcessInspecion();
            processInspecion.setCarrierId(bizDaywork.getId());

            //根据报工信息查询序检查信息
            List<BizProcessInspecion> processInspecions = bizProcessInspecionService.query().eq("daywork_id",bizDaywork.getId()).orderByDesc("create_time").list();
            if(processInspecions.size()>0) {
                BizProcessInspecion inspecion = processInspecions.get(0);
                if (inspecion.getStatus() != 1) {
                    perform = false;
                }
                //如果存在判断是是否有当前序的完成
                else {
                    perform = true;
                }
            }
        }
        return perform;
    }

    @Log(title = "检测箱号", businessType = BusinessType.SELECT)
    @PostMapping("/checkCarrier")
    public AjaxResult checkCarrier(@RequestBody BizDaywork bizDaywork) {
        // 检测新箱码是否已被使用
        BizCarrier curCarrier = bizCarrierService.getOne(new QueryWrapper<BizCarrier>().eq("id", bizDaywork.getNewCarrierId()));
        BizCarrierCategory carrierCategory = bizCarrierService.getCarrierCategory(curCarrier.getCategoryId());
        List<BizDayworkCarrier> dayworkCarrierList = bizDayworkCarrierService.query().eq("is_changed", 0).eq("carrier_id", bizDaywork.getNewCarrierId()).list();
//        if (dayworkCarrierList.size() > 0 && curCarrier.getIsAllowMore() == 0) { // 单独通过箱子的字段判断
//            return error("该载具已被使用，未解绑无法使用");
//        }
        if (dayworkCarrierList.size() > 0 && carrierCategory.getIsAllowMore() == 0) { // 通过类别判断
            return error("该载具已被使用，未解绑无法使用");
        }
        if (bizDaywork.getDayworkCarriers().stream().anyMatch(v -> v.getCarrierId().equals(bizDaywork.getNewCarrierId()))) {
            return error("已使用该载具");
        }
        BizCarrier carrier = new BizCarrier();
        carrier.setId(bizDaywork.getNewCarrierId());
        BizCarrier bizCarrier = bizCarrierService.getList(carrier).get(0);
        List<BizCarrierReject> carrierRejects = bizCarrierRejectService.query().eq("carrier_id",bizDaywork.getNewCarrierId()).orderByDesc("create_time").list();
        if (!carrierRejects.isEmpty() && carrierRejects.get(0).getIsAbandoned() == 1) {
            return error("该载具已被废弃");
        }
        // 检测新箱和旧箱是否是一个类型
        if (bizDaywork.getDayworkCarriers().size() == 0) {
            return success(bizCarrier);
        }
        List<BizCarrier> oldCarriers = bizCarrierService.query().in("id", bizDaywork.getDayworkCarriers().isEmpty() ? Collections.singletonList(0L) : bizDaywork.getDayworkCarriers().stream().map(BizDayworkCarrier::getCarrierId).collect(Collectors.toList())).list();
        List<BizCarrierCategory> categories = bizCarrierCategoryService.query().in("id", oldCarriers.isEmpty() ? Collections.singletonList(0L) : oldCarriers.stream().map(BizCarrier::getCategoryId).collect(Collectors.toList())).list();
        if (categories.stream().anyMatch(v -> !v.getIsAllowMore().equals(carrierCategory.getIsAllowMore()))) {
            return error("该载具与已有载具类型不同。");
        }
        // 检测是否有一个铁箱不该扫一个新箱号
        if (categories.stream().allMatch(v -> v.getIsAllowMore().equals(1))) {
            return error("已有铁箱载具，不需要添加其他载具。");
        }
        return success(bizCarrier);
    }
}
