package cn.ezhizao.project.business.product.controller;

import java.util.List;
import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;

import cn.ezhizao.common.utils.poi.ExcelUtil;
import cn.ezhizao.framework.aspectj.lang.annotation.Log;
import cn.ezhizao.framework.aspectj.lang.enums.BusinessType;
import cn.ezhizao.framework.web.controller.BaseController;
import cn.ezhizao.framework.web.domain.AjaxResult;
import cn.ezhizao.framework.web.page.TableDataInfo;
import cn.ezhizao.project.business.product.domain.BizDayworkItemReject;
import cn.ezhizao.project.business.product.service.IBizDayworkItemRejectService;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * 废品原因Controller
 *
 * @author ezhizao
 * @date 2023-12-28
 */
@RestController
@RequestMapping("/business/reject")
public class BizDayworkItemRejectController extends BaseController
{
    @Resource
    private IBizDayworkItemRejectService bizDayworkItemRejectService;

    /**
     * 查询废品原因列表
     */
    @PreAuthorize("@ss.hasPermi('business:reject:list')")
    @GetMapping("/list")
    public TableDataInfo list(BizDayworkItemReject bizDayworkItemReject) throws NoSuchFieldException, IllegalAccessException
    {
        setTenantId(bizDayworkItemReject);
        startPage();
        List<BizDayworkItemReject> list = bizDayworkItemRejectService.getList(bizDayworkItemReject);
        return getDataTable(list);
    }

    /**
     * 导出废品原因列表
     */
    @PreAuthorize("@ss.hasPermi('business:reject:export')")
    @Log(title = "废品原因", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, BizDayworkItemReject bizDayworkItemReject) throws NoSuchFieldException, IllegalAccessException
    {
        setTenantId(bizDayworkItemReject);
        List<BizDayworkItemReject> list = bizDayworkItemRejectService.getList(bizDayworkItemReject);
        ExcelUtil<BizDayworkItemReject> util = new ExcelUtil<BizDayworkItemReject>(BizDayworkItemReject.class);
        util.exportExcel(response, list, "废品原因数据");
    }

    /**
     * 获取废品原因详细信息
     */
    @PreAuthorize("@ss.hasPermi('business:reject:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(bizDayworkItemRejectService.getById(id));
    }

    /**
     * 新增废品原因
     */
    @PreAuthorize("@ss.hasPermi('business:reject:add')")
    @Log(title = "废品原因", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody BizDayworkItemReject bizDayworkItemReject) throws NoSuchFieldException, IllegalAccessException
    {
        setTenantId(bizDayworkItemReject);
        return toAjax(bizDayworkItemRejectService.save(bizDayworkItemReject));
    }

    /**
     * 修改废品原因
     */
    @PreAuthorize("@ss.hasPermi('business:reject:edit')")
    @Log(title = "废品原因", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody BizDayworkItemReject bizDayworkItemReject) throws NoSuchFieldException, IllegalAccessException
    {
        setTenantId(bizDayworkItemReject);
        return toAjax(bizDayworkItemRejectService.updateById(bizDayworkItemReject));
    }

    /**
     * 删除废品原因
     */
    @PreAuthorize("@ss.hasPermi('business:reject:remove')")
    @Log(title = "废品原因", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable List<Long> ids)
    {
        return toAjax(bizDayworkItemRejectService.removeBatchByIds(ids));
    }
}
