package cn.ezhizao.project.business.product.controller;

import java.util.List;
import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;

import cn.ezhizao.common.utils.poi.ExcelUtil;
import cn.ezhizao.framework.aspectj.lang.annotation.Log;
import cn.ezhizao.framework.aspectj.lang.enums.BusinessType;
import cn.ezhizao.framework.web.controller.BaseController;
import cn.ezhizao.framework.web.domain.AjaxResult;
import cn.ezhizao.framework.web.page.TableDataInfo;
import cn.ezhizao.project.business.product.domain.BizFurnaceNoInfo;
import cn.ezhizao.project.business.product.service.IBizFurnaceNoInfoService;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;


/**
 * 企业租户Controller
 *
 * @author ezhizao
 * @date 2024-01-30
 */
@RestController
@RequestMapping("/business/info")
public class BizFurnaceNoInfoController extends BaseController
{
    @Resource
    private IBizFurnaceNoInfoService bizFurnaceNoInfoService;

    /**
     * 查询企业租户列表
     */
//    @PreAuthorize("@ss.hasPermi('business:info:list')")
    @GetMapping("/list")
    public TableDataInfo list(BizFurnaceNoInfo bizFurnaceNoInfo) throws NoSuchFieldException, IllegalAccessException
    {
//        setTenantId(bizFurnaceNoInfo);
//        startPage();
        List<BizFurnaceNoInfo> list = bizFurnaceNoInfoService.getList(bizFurnaceNoInfo);
        return getDataTable(list);
    }

    /**
     * 导出企业租户列表
     */
//    @PreAuthorize("@ss.hasPermi('business:info:export')")
    @Log(title = "企业租户", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, BizFurnaceNoInfo bizFurnaceNoInfo) throws NoSuchFieldException, IllegalAccessException
    {
        setTenantId(bizFurnaceNoInfo);
        List<BizFurnaceNoInfo> list = bizFurnaceNoInfoService.getList(bizFurnaceNoInfo);
        ExcelUtil<BizFurnaceNoInfo> util = new ExcelUtil<BizFurnaceNoInfo>(BizFurnaceNoInfo.class);
        util.exportExcel(response, list, "企业租户数据");
    }

    /**
     * 获取企业租户详细信息
     */
//    @PreAuthorize("@ss.hasPermi('business:info:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(bizFurnaceNoInfoService.getById(id));
    }

    /**
     * 新增企业租户
     */
//    @PreAuthorize("@ss.hasPermi('business:info:add')")
    @Log(title = "企业租户", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody BizFurnaceNoInfo bizFurnaceNoInfo) throws NoSuchFieldException, IllegalAccessException
    {
        setTenantId(bizFurnaceNoInfo);
        return toAjax(bizFurnaceNoInfoService.save(bizFurnaceNoInfo));
    }

    /**
     * 修改企业租户
     */
//    @PreAuthorize("@ss.hasPermi('business:info:edit')")
    @Log(title = "企业租户", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody BizFurnaceNoInfo bizFurnaceNoInfo) throws NoSuchFieldException, IllegalAccessException
    {
        setTenantId(bizFurnaceNoInfo);
        return toAjax(bizFurnaceNoInfoService.updateById(bizFurnaceNoInfo));
    }

    /**
     * 删除企业租户
     */
//    @PreAuthorize("@ss.hasPermi('business:info:remove')")
    @Log(title = "企业租户", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable List<Long> ids)
    {
        return toAjax(bizFurnaceNoInfoService.removeBatchByIds(ids));
    }
}
