package cn.ezhizao.project.business.product.controller;

import java.util.List;
import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;

import cn.ezhizao.common.utils.poi.ExcelUtil;
import cn.ezhizao.framework.aspectj.lang.annotation.Log;
import cn.ezhizao.framework.aspectj.lang.enums.BusinessType;
import cn.ezhizao.framework.web.controller.BaseController;
import cn.ezhizao.framework.web.domain.AjaxResult;
import cn.ezhizao.framework.web.page.TableDataInfo;
import cn.ezhizao.project.business.product.domain.BizLot;
import cn.ezhizao.project.business.product.domain.BizLotTechnologicalProcess;
import cn.ezhizao.project.business.product.domain.BizLotTechnologicalProcessDetail;
import cn.ezhizao.project.business.product.service.*;
import cn.ezhizao.project.business.technologicalProcess.service.IBizTechnologicalProcessService;
import cn.ezhizao.project.business.technologicalProcessDetail.domain.BizTechnologicalProcessDetail;
import cn.ezhizao.project.business.technologicalProcessDetail.service.IBizTechnologicalProcessDetailService;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;


/**
 * 批次号Controller
 *
 * @author ezhizao
 * @date 2023-12-04
 */
@RestController
@RequestMapping("/business/lot")
public class BizLotController extends BaseController {
    @Resource
    private IBizLotService bizLotService;

    @Resource
    private IBizTechnologicalProcessDetailService bizTechnologicalProcessDetailService;

    @Resource
    private IBizTechnologicalProcessService bizTechnologicalProcessService;

    @Resource
    private IBizLotTechnologicalProcessDetailService bizLotTechnologicalProcessDetailService;
    @Resource
    private IBizLotTechnologicalProcessService bizLotTechnologicalProcessService;

    /**
     * 查询批次号列表
     */
//    @PreAuthorize("@ss.hasPermi('business:lot:list')")
    @GetMapping("/list")
    public TableDataInfo list(BizLot bizLot) throws NoSuchFieldException, IllegalAccessException {
        List<BizLot> list = bizLotService.getList(bizLot);
        return getDataTable(list);
    }
    /**
     * 查询该计划单当前工段废品回用的批次
     */
    @GetMapping("/getAbnormalityLot")
    public TableDataInfo getAbnormalityLot(BizLot bizLot) throws NoSuchFieldException, IllegalAccessException {
        //该工段显示开始新批次按钮时，不传flag，当选择批次号时，传flag
        List<BizLot> list = bizLotService.getWasteRecyclingLot(bizLot);
        return getDataTable(list);
    }
    /**
     * 查询该批次对应的工序
     */
    @GetMapping("/getProcessListByLot")
    public AjaxResult getProcessListByLot(BizLot bizLot) throws NoSuchFieldException, IllegalAccessException {
            //该批是否废品回用
            if(bizLot.getIsWasteRecycling() == 1 || bizLot.getIsAmend() == 1 ) {
//                QueryWrapper<BizLotTechnologicalProcess> queryWrapper = new QueryWrapper<>();
//                queryWrapper.eq("lot_id",bizLot.getId());
//                queryWrapper.eq("is_stop",0);
//                BizLotTechnologicalProcess wasteRecycling = bizLotTechnologicalProcessService.getOne(queryWrapper);
                QueryWrapper<BizLotTechnologicalProcessDetail> wrapper = new QueryWrapper<>();
                wrapper.eq("lot_technological_process_id",bizLot.getTechnologicalProcessId());
                //去掉备料的序
                wrapper.ne("process_id",0);
                wrapper.ne("process_id",1);
                wrapper.orderByAsc("process_step_number");
                List<BizLotTechnologicalProcessDetail> list = bizLotTechnologicalProcessDetailService.list(wrapper);
                return success(list);
            }
            //查原始工艺工序表
            else {
                QueryWrapper<BizTechnologicalProcessDetail> wrapper = new QueryWrapper<>();
                wrapper.eq("technological_process_id",bizLot.getTechnologicalProcessId());
                wrapper.ne("process_id",0);
                wrapper.ne("process_id",1);
                wrapper.ne("tenant_id", 9);
                wrapper.orderByAsc("process_step_number");
                return success(bizTechnologicalProcessDetailService.list(wrapper));
            }
        }


    /**
     * 导出批次号列表
     */
//    @PreAuthorize("@ss.hasPermi('business:lot:export')")
    @Log(title = "批次号", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, BizLot bizLot) throws NoSuchFieldException, IllegalAccessException {
        setTenantId(bizLot);
        List<BizLot> list = bizLotService.getList(bizLot);
        ExcelUtil<BizLot> util = new ExcelUtil<BizLot>(BizLot.class);
        util.exportExcel(response, list, "批次号数据");
    }

    /**
     * 获取批次号详细信息
     */
//    @PreAuthorize("@ss.hasPermi('business:lot:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id) {
        return success(bizLotService.getById(id));
    }

    /**
     * 新增批次号
     */
//    @PreAuthorize("@ss.hasPermi('business:lot:add')")
    @Log(title = "批次号", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody BizLot bizLot) throws NoSuchFieldException, IllegalAccessException {
        setTenantId(bizLot);
        return toAjax(bizLotService.save(bizLot));
    }

    /**
     * 修改批次号
     */
//    @PreAuthorize("@ss.hasPermi('business:lot:edit')")
    @Log(title = "批次号", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody BizLot bizLot) throws NoSuchFieldException, IllegalAccessException {
        setTenantId(bizLot);
        return toAjax(bizLotService.updateById(bizLot));
    }

    /**
     * 删除批次号
     */
//    @PreAuthorize("@ss.hasPermi('business:lot:remove')")
    @Log(title = "批次号", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable List<Long> ids) {
        return toAjax(bizLotService.removeBatchByIds(ids));
    }
}
