package cn.ezhizao.project.business.product.controller;

import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;
import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;

import cn.ezhizao.common.utils.poi.ExcelUtil;
import cn.ezhizao.framework.aspectj.lang.annotation.Log;
import cn.ezhizao.framework.aspectj.lang.enums.BusinessType;
import cn.ezhizao.framework.web.controller.BaseController;
import cn.ezhizao.framework.web.domain.AjaxResult;
import cn.ezhizao.framework.web.page.TableDataInfo;
import cn.ezhizao.project.business.product.domain.*;
import cn.ezhizao.project.business.product.mapper.BizDayworkCarrierMapper;
import cn.ezhizao.project.business.product.service.*;
import cn.ezhizao.project.business.technologicalProcessDetail.domain.BizTechnologicalProcessDetail;
import cn.ezhizao.project.business.technologicalProcessDetail.service.IBizTechnologicalProcessDetailService;
import cn.ezhizao.project.system.domain.SysUser;
import cn.ezhizao.project.system.service.ISysUserService;
import com.baomidou.mybatisplus.annotation.TableField;
import org.apache.xmlbeans.impl.soap.Detail;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;


/**
 * 序检主Controller
 *
 * @author ezhizao
 * @date 2024-04-24
 */
@RestController
@RequestMapping("/business/inspecion")
public class BizProcessInspecionController extends BaseController {
    @Resource
    private IBizProcessInspecionService bizProcessInspecionService;

    @Resource
    private ISysUserService sysUserService;

    @Resource
    private IBizDayworkCarrierService bizDayworkCarrierService;
    @Resource
    private IBizLotTechnologicalProcessDetailService bizLotTechnologicalProcessDetailService;

    @Resource
    private IBizTechnologicalProcessDetailService bizTechnologicalProcessDetailService;

    @Resource
    private IBizDayworkItemRejectService bizDayworkItemRejectService;

    @Resource
    private IBizDayworkItemConsultService bizDayworkItemConsultService;

    @Resource
    private IBizDayworkService bizDayworkService;

    @Resource
    private IBizProductService bizProductService;

    @Resource
    private IBizDayworkItemService bizDayworkItemService;

    @Resource
    private IBizLotService bizLotService;

    @Resource
    private IBizProductService productService;
    @Resource
    private IBizReturnReceiptDetailService bizReturnReceiptDetailService;
    @Resource
    private IBizProductionPlanDetailService productionPlanDetailService;

    /**
     * 查询序检主列表
     */
//    @PreAuthorize("@ss.hasPermi('business:inspecion:list')")
    @PostMapping("/list")
    public TableDataInfo list(@RequestBody BizProcessInspecion bizProcessInspecion) throws NoSuchFieldException, IllegalAccessException {

        if(bizProcessInspecion.getStartTime()!=null) {
            Date startTimeDate = bizProcessInspecion.getStartTime();
            // 使用 SimpleDateFormat 格式化 Date 对象为特定格式的字符串
            SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd", Locale.getDefault());
            String time= dateFormat.format(startTimeDate);

            bizProcessInspecion.setStartTimeString(time+ " 00:00:00");
            bizProcessInspecion.setEndTimeString(time+ " 23:59:59");
        }


        List<BizProcessInspecion> list = bizProcessInspecionService.getList(bizProcessInspecion);
        return getDataTable(list);
    }

    /**
     * 查询序检主列表(外协序检)
     */
//    @PreAuthorize("@ss.hasPermi('business:inspecion:list')")
    @PostMapping("/outsourcedlist")
    public TableDataInfo outsourcedlist(@RequestBody BizProcessInspecion bizProcessInspecion) throws NoSuchFieldException, IllegalAccessException {
        if(bizProcessInspecion.getStartTime()!=null) {
            Date startTimeDate = bizProcessInspecion.getStartTime();
            // 使用 SimpleDateFormat 格式化 Date 对象为特定格式的字符串
            SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd", Locale.getDefault());
            String time= dateFormat.format(startTimeDate);

            bizProcessInspecion.setStartTimeString(time+ " 00:00:00");
            bizProcessInspecion.setEndTimeString(time+ " 23:59:59");
        }
        List<BizProcessInspecion> list = bizProcessInspecionService.getOutsourcedlist(bizProcessInspecion);
        return getDataTable(list);
    }

    /**
     * 查询产品信息
     */
    @PostMapping("/getProductConsult")
    public AjaxResult getProductConsult(@RequestBody BizLot lot) throws NoSuchFieldException, IllegalAccessException {
        BizProduct product = bizProductService.getById(lot.getProductId());
        if (product == null) {
            throw (new RuntimeException("该产品不存在"));
        }

        SysUser user = sysUserService.selectUserById(product.getTechnicianId());
        product.setTechnicianName(user == null ? "" : user.getNickName());

        return success(product);
    }

    /**
     * 查询序捡详情
     *
     * @param processInspecion
     * @return
     * @throws NoSuchFieldException
     * @throws IllegalAccessException
     */
    @PostMapping("/selectInspecion")
    public AjaxResult selectInspecion(@RequestBody BizProcessInspecion processInspecion) throws NoSuchFieldException, IllegalAccessException {
        List<BizDayworkItemConsult> itemConsults = bizDayworkItemConsultService.query().eq("process_inspection_id", processInspecion.getId()).eq("is_process_inspection", 1).list();
        List<BizDayworkItemReject> itemRejects = bizDayworkItemRejectService.query().eq("process_inspection_id", processInspecion.getId()).list();
        processInspecion.setDayworkItemConsults(itemConsults);
        processInspecion.setDayworkItemRejects(itemRejects);

        BizDayworkItem dayworkItem = bizDayworkItemService.selById(processInspecion.getDayworkItemId());

        BizLot lot = new BizLot();
        lot.setId(processInspecion.getLotId());
        lot = bizLotService.getAllList(lot).get(0);

        if (lot.getIsAmend() == 1 || lot.getIsWasteRecycling() == 1) {
            BizLotTechnologicalProcessDetail process = bizLotTechnologicalProcessDetailService.getById(dayworkItem.getTechnologicalProcessDetailId());
            lot.setProcessAlias(process.getProcessAlias());
        } else {
            BizTechnologicalProcessDetail process = bizTechnologicalProcessDetailService.getById(dayworkItem.getTechnologicalProcessDetailId());
            lot.setProcessAlias(process.getProcessAlias());
        }

        lot.setAllCarrierName(dayworkItem.getCarrierName());
        lot.setDayworkItemId(dayworkItem.getId());
        lot.setCarrierCode(dayworkItem.getCarrierCode());
        lot.setCarrierId(dayworkItem.getCarrierId());
        lot.setPudName(dayworkItem.getProdNum() == 0 ? lot.getProductionQuantity() : dayworkItem.getProdNum());
        processInspecion.setLot(lot);
        return success(processInspecion);
    }


    @PostMapping("/selectOutsourcedInspecion")
    public AjaxResult selectOutsourcedInspecion(@RequestBody BizProcessInspecion processInspecion) throws NoSuchFieldException, IllegalAccessException {
        List<BizDayworkItemConsult> itemConsults = bizDayworkItemConsultService.query().eq("process_inspection_id", processInspecion.getId()).eq("is_outsourced_inspection", 1).list();
        List<BizDayworkItemReject> itemRejects = bizDayworkItemRejectService.query().eq("process_inspection_id", processInspecion.getId()).list();
        processInspecion.setDayworkItemConsults(itemConsults);
        processInspecion.setDayworkItemRejects(itemRejects);

        BizDayworkItem dayworkItem = bizDayworkItemService.selById(processInspecion.getDayworkItemId());

        BizLot lot = new BizLot();
        lot.setId(processInspecion.getLotId());
        lot = bizLotService.getAllList(lot).get(0);

        if (lot.getIsAmend() == 1 || lot.getIsWasteRecycling() == 1) {
            BizLotTechnologicalProcessDetail process = bizLotTechnologicalProcessDetailService.getById(dayworkItem.getTechnologicalProcessDetailId());
            lot.setProcessAlias(process.getProcessAlias());
        } else {
            BizTechnologicalProcessDetail process = bizTechnologicalProcessDetailService.getById(dayworkItem.getTechnologicalProcessDetailId());
            lot.setProcessAlias(process.getProcessAlias());
        }

        lot.setAllCarrierName(dayworkItem.getCarrierName());
        lot.setDayworkItemId(dayworkItem.getId());
        lot.setCarrierCode(dayworkItem.getCarrierCode());
        lot.setCarrierId(dayworkItem.getCarrierId());
        lot.setPudName(dayworkItem.getProdNum() == 0 ? lot.getProductionQuantity() : dayworkItem.getProdNum());
        processInspecion.setLot(lot);
        return success(processInspecion);
    }

    /**
     * 新增外协序检信息
     */
    @PostMapping("/saveOutsourcedInspecion")
    @Transactional
    public AjaxResult saveOutsourcedInspecion(@RequestBody BizProcessInspecion processInspecion) throws NoSuchFieldException, IllegalAccessException {
        //根据lot信息保存序检表
        BizDaywork daywork = bizDayworkService.query().eq("lot_code", processInspecion.getLot().getLotCode()).one();
        BizProduct product = bizProductService.getById(processInspecion.getLot().getProductId());
        SysUser technicianUser = sysUserService.selectUserById(product.getTechnicianId());

        SysUser user = sysUserService.selectUserById(processInspecion.getUser().getUserId());
        BizDayworkItem dayworkItem = bizDayworkItemService.getById(processInspecion.getLot().getDayworkItemId());
        System.out.println("DayworkID:" + dayworkItem.getDayworkId());
        System.out.println("getTechnologicalProcessDetailId:" + dayworkItem.getTechnologicalProcessDetailId());
        BizReturnReceiptDetail bizReturnReceiptDetail = bizReturnReceiptDetailService.query().eq("daywork_id", dayworkItem.getDayworkId()).eq("technological_process_detail_id", dayworkItem.getTechnologicalProcessDetailId()).eq("deleted", 0).one();
        System.out.println("回收:" + bizReturnReceiptDetail);
        //根据dayworkItem的工序ID和dayworkId查询回收Id
        processInspecion.setUserId(user == null ? 0L : user.getUserId());
        processInspecion.setNickName(user == null ? "" : user.getNickName());
        processInspecion.setReturnReceiptDetailId(bizReturnReceiptDetail.getId());
        processInspecion.setCarrierId(processInspecion.getLot().getCarrierId());
        processInspecion.setCarrierCode(processInspecion.getLot().getCarrierCode());
        processInspecion.setDayworkId(daywork.getId());
        processInspecion.setProductionPlanId(daywork.getProductionPlanId());
        processInspecion.setDayworkItemId(dayworkItem.getId());
        processInspecion.setProductionPlanDetailId(daywork.getProductionPlanDetailId());
        processInspecion.setProductionPlanDetailSubDetailId(daywork.getProductionPlanDetailSubDetailId());
        processInspecion.setLotId(processInspecion.getLot().getId());
        processInspecion.setLotCode(processInspecion.getLot().getLotCode());
        processInspecion.setDeptId(daywork.getDeptId());
        processInspecion.setProductId(processInspecion.getLot().getProductId());
        processInspecion.setTechnologicalProcessId(processInspecion.getLot().getTechnologicalProcessId());
        if (processInspecion.getStartTime() == null) {
            processInspecion.setStartTime(new Date());
        }

        if (processInspecion.getStatus() != 0) {
            processInspecion.setEndTime(new Date());

        }

        //保存
        bizProcessInspecionService.saveOrUpdate(processInspecion);

        //保存废品信息
        List<BizDayworkItemReject> rejects = processInspecion.getDayworkItemRejects();
        //保存前先删除原先废品信息
        BizDayworkItemReject rej = new BizDayworkItemReject();
        rej.setProcessInspectionId(processInspecion.getId());
        bizDayworkItemRejectService.removeReject(rej);
        if (rejects != null && rejects.size() > 0) {
            for (BizDayworkItemReject reject : rejects) {
                reject.setId(null);
                reject.setUserId(user == null ? 0L : user.getUserId());
                reject.setProcessInspectionId(processInspecion.getId());
                reject.setDayworkId(0L);
                reject.setProductionPlanId(daywork.getProductionPlanId());
                reject.setProductionPlanDetailId(daywork.getProductionPlanDetailId());
                reject.setProductionPlanDetailSubDetailId(daywork.getProductionPlanDetailSubDetailId());
                reject.setLotId(processInspecion.getLot().getId());
                reject.setDayworkItemId(dayworkItem.getId());
            }
        }
        bizDayworkItemRejectService.saveBatch(rejects);
        //保存咨询信息

        List<BizDayworkItemConsult> consults = processInspecion.getDayworkItemConsults();
        if (consults != null && consults.size() > 0) {
            for (BizDayworkItemConsult consult : consults) {
                consult.setUserId(user == null ? 0L : user.getUserId());
                consult.setNickName(user == null ? "" : user.getNickName());
                consult.setDayworkId(daywork.getId());
                consult.setProcessInspectionId(processInspecion.getId());
                consult.setProductionPlanId(daywork.getProductionPlanId());
                consult.setProductionPlanDetailId(daywork.getProductionPlanDetailId());
                consult.setProductionPlanDetailSubDetailId(daywork.getProductionPlanDetailSubDetailId());
                consult.setLotId(processInspecion.getLot().getId());
                consult.setLotCode(processInspecion.getLot().getLotCode());
                consult.setDayworkId(daywork.getId());
                consult.setDayworkItemId(dayworkItem.getId());
                consult.setProductId(processInspecion.getLot().getProductId());
                consult.setProductDescription(processInspecion.getLot().getProductDescription());
                consult.setTechnologicalProcessId(processInspecion.getLot().getTechnologicalProcessId());
                consult.setTechnologicalProcessDetailId(dayworkItem.getTechnologicalProcessDetailId());
                consult.setProcessId(dayworkItem.getProcessId());
                consult.setProcessAlias(processInspecion.getLot().getProcessAlias());
                consult.setIsOutsourcedInspection(1);
                consult.setTechnicianId(technicianUser == null ? 0L : technicianUser.getUserId());
            }
        }

        //修改daywaork信息
        dayworkItem.setIsProcessInspection(1);
        return success(bizDayworkItemService.saveOrUpdate(dayworkItem) && bizDayworkItemConsultService.saveOrUpdateBatch(consults));
    }

    /**
     * 新增序检信息
     */
    @PostMapping("/saveProcessInspecion")
    @Transactional
    public AjaxResult saveProcessInspecion(@RequestBody BizProcessInspecion processInspecion) throws NoSuchFieldException, IllegalAccessException {
        //根据lot信息保存序检表
        BizDaywork daywork = bizDayworkService.query().eq("lot_code", processInspecion.getLot().getLotCode()).one();
        BizProduct product = bizProductService.getById(processInspecion.getLot().getProductId());
        SysUser technicianUser = sysUserService.selectUserById(product.getTechnicianId());

        SysUser user = sysUserService.selectUserById(processInspecion.getUser().getUserId());
        BizDayworkItem dayworkItem = bizDayworkItemService.getById(processInspecion.getLot().getDayworkItemId());
        processInspecion.setUserId(user == null ? 0L : user.getUserId());
        processInspecion.setNickName(user == null ? "" : user.getNickName());
        processInspecion.setCarrierId(processInspecion.getLot().getCarrierId());
        processInspecion.setCarrierCode(processInspecion.getLot().getCarrierCode());
        processInspecion.setDayworkId(daywork.getId());
        processInspecion.setProductionPlanId(daywork.getProductionPlanId());
        processInspecion.setDayworkItemId(dayworkItem.getId());
        processInspecion.setProductionPlanDetailId(daywork.getProductionPlanDetailId());
        processInspecion.setProductionPlanDetailSubDetailId(daywork.getProductionPlanDetailSubDetailId());
        processInspecion.setLotId(processInspecion.getLot().getId());
        processInspecion.setLotCode(processInspecion.getLot().getLotCode());
        processInspecion.setDeptId(daywork.getDeptId());
        processInspecion.setProductId(processInspecion.getLot().getProductId());
        processInspecion.setTechnologicalProcessId(processInspecion.getLot().getTechnologicalProcessId());
        if (processInspecion.getStartTime() == null) {
            processInspecion.setStartTime(new Date());
        }

        if (processInspecion.getStatus() != 0) {
            processInspecion.setEndTime(new Date());

        }

        //保存
        bizProcessInspecionService.saveOrUpdate(processInspecion);

        //保存废品信息
        List<BizDayworkItemReject> rejects = processInspecion.getDayworkItemRejects();
        //保存前先删除原先废品信息
        BizDayworkItemReject rej = new BizDayworkItemReject();
        rej.setProcessInspectionId(processInspecion.getId());
        bizDayworkItemRejectService.removeReject(rej);
        if (rejects != null && rejects.size() > 0) {
            for (BizDayworkItemReject reject : rejects) {
                reject.setId(null);
                reject.setUserId(user == null ? 0L : user.getUserId());
                reject.setProcessInspectionId(processInspecion.getId());
                reject.setDayworkId(0L);
                reject.setProductionPlanId(daywork.getProductionPlanId());
                reject.setProductionPlanDetailId(daywork.getProductionPlanDetailId());
                reject.setProductionPlanDetailSubDetailId(daywork.getProductionPlanDetailSubDetailId());
                reject.setLotId(processInspecion.getLot().getId());
                reject.setDayworkItemId(dayworkItem.getId());
            }
        }
        bizDayworkItemRejectService.saveBatch(rejects);
        //保存咨询信息

        List<BizDayworkItemConsult> consults = processInspecion.getDayworkItemConsults();
        if (consults != null && consults.size() > 0) {
            for (BizDayworkItemConsult consult : consults) {
                consult.setUserId(user == null ? 0L : user.getUserId());
                consult.setNickName(user == null ? "" : user.getNickName());
                consult.setDayworkId(daywork.getId());
                consult.setProcessInspectionId(processInspecion.getId());
                consult.setProductionPlanId(daywork.getProductionPlanId());
                consult.setProductionPlanDetailId(daywork.getProductionPlanDetailId());
                consult.setProductionPlanDetailSubDetailId(daywork.getProductionPlanDetailSubDetailId());
                consult.setLotId(processInspecion.getLot().getId());
                consult.setLotCode(processInspecion.getLot().getLotCode());
                consult.setDayworkId(daywork.getId());
                consult.setDayworkItemId(dayworkItem.getId());
                consult.setProductId(processInspecion.getLot().getProductId());
                consult.setProductDescription(processInspecion.getLot().getProductDescription());
                consult.setTechnologicalProcessId(processInspecion.getLot().getTechnologicalProcessId());
                consult.setTechnologicalProcessDetailId(dayworkItem.getTechnologicalProcessDetailId());
                consult.setProcessId(dayworkItem.getProcessId());
                consult.setProcessAlias(processInspecion.getLot().getProcessAlias());
                consult.setIsProcessInspection(1);
                consult.setTechnicianId(technicianUser == null ? 0L : technicianUser.getUserId());
            }
        }

        //修改daywaork信息
        dayworkItem.setIsProcessInspection(1);
        return success(bizDayworkItemService.saveOrUpdate(dayworkItem) && bizDayworkItemConsultService.saveOrUpdateBatch(consults));
    }


    /**
     * 根据箱号查询是否为一箱多批
     */
    @PostMapping("/getCarrierInfo")
    public AjaxResult getCarrierInfo(@RequestBody BizDayworkCarrier dayworkCarrier) throws NoSuchFieldException, IllegalAccessException {
//        List<BizDayworkCarrier> list = bizDayworkCarrierService.query().eq("carrier_code",dayworkCarrier.getCarrierCode()).eq("deleted",0).eq("is_changed",0).list();
        List<BizDayworkCarrier> list = bizDayworkCarrierService.getList(dayworkCarrier);
        if (list.size() == 0) {
            throw new RuntimeException("该箱号未绑定任何批次");
        }

        return success(list);
    }

    /**
     * 根据箱号查询批次信息
     */
//    @PreAuthorize("@ss.hasPermi('business:inspecion:list')")
    @PostMapping("/getLotOutsourcedInfo")
    public AjaxResult getLotOutsourcedInfo(@RequestBody BizDayworkCarrier dayworkCarrier) throws NoSuchFieldException, IllegalAccessException {
        List<String> processCodes = dayworkCarrier.getProcessCode();
        System.out.println("查询工序编码：processCodes" + processCodes);
        //根据箱号查询当前绑定信息
        dayworkCarrier = bizDayworkCarrierService.getByCarrierCode(dayworkCarrier);
        if (dayworkCarrier == null) {
            throw new RuntimeException("该箱号未绑定任何批次");
        }
        System.out.println("请求信息：dayworkCarrier" + dayworkCarrier);

        //根据dayworkId查询外协回收信息是否有该批次
        BizReturnReceiptDetail bizReturnReceiptDetail = new BizReturnReceiptDetail();
        bizReturnReceiptDetail.setDayworkId(dayworkCarrier.getDayworkId());
        List<BizReturnReceiptDetail> detailList = bizReturnReceiptDetailService.getList(bizReturnReceiptDetail);
        System.out.println("查询是否有该批次：detailList" + detailList);
        if (detailList.size() == 0) {
            throw new RuntimeException("该批次不是外协回收批次");
        }

        detailList = detailList.stream()
                .filter(detail -> processCodes.stream().anyMatch(l -> l.trim().equals(detail.getProcessCode())))
                .collect(Collectors.toList());
        if (detailList.size() == 0) {
            throw new RuntimeException("该批次不需要外协检查");
        }
        //如果有多道工序，则根据工序的排序找到最后一条
        if (detailList.size() > 1) {
            //按照工序排序
            detailList.sort(Comparator.comparing(BizReturnReceiptDetail::getProcessStepNumber, Comparator.reverseOrder()));
        }
        //根据工序id查询报工信息
        BizDaywork daywork = new BizDaywork();
        daywork.setId(detailList.get(0).getDayworkId());
        daywork.setTechnologicalProcessDetailId(detailList.get(0).getTechnologicalProcessDetailId());

        List<BizDayworkItem> itemList = bizDayworkItemService.getByDaywork(daywork);
        if (itemList.size() == 0) {
            throw new RuntimeException("该批次未开始加工");
        }
        BizDayworkItem item = itemList.get(0);
        BizProductionPlanDetail productionPlanDetail = productionPlanDetailService.getById(item.getProductionPlanDetailId());
        BizProduct product = productService.getById(productionPlanDetail.getProductId());
        SysUser user = sysUserService.selectUserById(product.getTechnicianId());
        if (user != null) {
            item.setTechnicianName(user.getNickName());
            item.setTechnicianId(user.getUserId());
        } else {
            item.setTechnicianName("");
            item.setTechnicianId(0L);
        }

        //查询是否为正常批次
        BizLot lot = new BizLot();
        lot.setId(item.getLotId());
        lot = bizLotService.getAllList(lot).get(0);
        if (lot.getIsWaste() == 1) {
            throw new RuntimeException("改批次已批废");
        }
        if (lot.getIsAmend() == 1 || lot.getIsWasteRecycling() == 1) {
            BizLotTechnologicalProcessDetail process = bizLotTechnologicalProcessDetailService.getById(item.getTechnologicalProcessDetailId());
            lot.setProcessAlias(process.getProcessAlias());
        } else {
            BizTechnologicalProcessDetail process = bizTechnologicalProcessDetailService.getById(item.getTechnologicalProcessDetailId());
            lot.setProcessAlias(process.getProcessAlias());
        }
        lot.setAllCarrierName(item.getCarrierName());
        lot.setDayworkItemId(item.getId());
        lot.setCarrierCode(dayworkCarrier.getCarrierCode());
        lot.setCarrierId(dayworkCarrier.getCarrierId());
        lot.setPudName(item.getProdNum() == 0 ? lot.getProductionQuantity() : item.getProdNum());
        return success(lot);
    }


    /**
     * 根据箱号查询批次信息
     */
//    @PreAuthorize("@ss.hasPermi('business:inspecion:list')")
    @PostMapping("/getLotInfo")
    public AjaxResult getLotInfo(@RequestBody BizDayworkCarrier dayworkCarrier) throws NoSuchFieldException, IllegalAccessException {
        //根据箱号查询当前绑定信息
        dayworkCarrier = bizDayworkCarrierService.getByCarrierCode(dayworkCarrier);
        if (dayworkCarrier == null) {
            throw new RuntimeException("该箱号未绑定任何批次");
        }

        //获取报工信息
        List<BizDayworkItem> itemList = bizDayworkItemService.getByDayworkId(dayworkCarrier.getDayworkId());
        if (itemList.size() == 0) {
            throw new RuntimeException("改批次未开始加工");
        }
        BizDayworkItem item = itemList.get(0);
        BizProductionPlanDetail productionPlanDetail = productionPlanDetailService.getById(item.getProductionPlanDetailId());
        BizProduct product = productService.getById(productionPlanDetail.getProductId());
        SysUser user = sysUserService.selectUserById(product.getTechnicianId());
        if (user != null) {
            item.setTechnicianName(user.getNickName());
            item.setTechnicianId(user.getUserId());
        } else {
            item.setTechnicianName("");
            item.setTechnicianId(0L);
        }

        //查询是否为正常批次
        BizLot lot = new BizLot();
        lot.setId(item.getLotId());
        lot = bizLotService.getAllList(lot).get(0);
        if (lot.getIsWaste() == 1) {
            throw new RuntimeException("改批次已批废");
        }
        if (lot.getIsAmend() == 1 || lot.getIsWasteRecycling() == 1) {
            BizLotTechnologicalProcessDetail process = bizLotTechnologicalProcessDetailService.getById(item.getTechnologicalProcessDetailId());
            lot.setProcessAlias(process.getProcessAlias());
        } else {
            BizTechnologicalProcessDetail process = bizTechnologicalProcessDetailService.getById(item.getTechnologicalProcessDetailId());
            lot.setProcessAlias(process.getProcessAlias());
        }
        lot.setAllCarrierName(item.getCarrierName());
        lot.setDayworkItemId(item.getId());
        lot.setCarrierCode(dayworkCarrier.getCarrierCode());
        lot.setCarrierId(dayworkCarrier.getCarrierId());
        lot.setPudName(item.getProdNum() == 0 ? lot.getProductionQuantity() : item.getProdNum());
        return success(lot);
    }


//

    /**
     * 导出序检主列表
     */
    @PreAuthorize("@ss.hasPermi('business:inspecion:export')")
    @Log(title = "序检主", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, BizProcessInspecion bizProcessInspecion) throws NoSuchFieldException, IllegalAccessException {
        setTenantId(bizProcessInspecion);
        List<BizProcessInspecion> list = bizProcessInspecionService.getList(bizProcessInspecion);
        ExcelUtil<BizProcessInspecion> util = new ExcelUtil<BizProcessInspecion>(BizProcessInspecion.class);
        util.exportExcel(response, list, "序检主数据");
    }

    /**
     * 获取序检主详细信息
     */
    @PreAuthorize("@ss.hasPermi('business:inspecion:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id) {
        return success(bizProcessInspecionService.getById(id));
    }

    /**
     * 新增序检主
     */
    @PreAuthorize("@ss.hasPermi('business:inspecion:add')")
    @Log(title = "序检主", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody BizProcessInspecion bizProcessInspecion) throws NoSuchFieldException, IllegalAccessException {
        setTenantId(bizProcessInspecion);
        return toAjax(bizProcessInspecionService.save(bizProcessInspecion));
    }

    /**
     * 修改序检主
     */
    @PreAuthorize("@ss.hasPermi('business:inspecion:edit')")
    @Log(title = "序检主", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody BizProcessInspecion bizProcessInspecion) throws NoSuchFieldException, IllegalAccessException {
        setTenantId(bizProcessInspecion);
        return toAjax(bizProcessInspecionService.updateById(bizProcessInspecion));
    }

    /**
     * 删除序检主
     */
    @PreAuthorize("@ss.hasPermi('business:inspecion:remove')")
    @Log(title = "序检主", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable List<Long> ids) {
        return toAjax(bizProcessInspecionService.removeBatchByIds(ids));
    }
}
