package cn.ezhizao.project.business.product.controller;

import cn.ezhizao.common.utils.SecurityUtils;
import cn.ezhizao.framework.aspectj.lang.annotation.Log;
import cn.ezhizao.framework.aspectj.lang.enums.BusinessType;
import cn.ezhizao.framework.web.controller.BaseController;
import cn.ezhizao.framework.web.domain.AjaxResult;
import cn.ezhizao.framework.web.page.TableDataInfo;
import cn.ezhizao.project.business.product.domain.*;
import cn.ezhizao.project.business.product.service.IBizDayworkService;
import cn.ezhizao.project.business.product.service.IBizProductionPlanDetailService;
import cn.ezhizao.project.business.resourceGroup.domain.BizProductionResourceGroup;
import cn.ezhizao.project.business.resourceGroup.domain.BizProductionResourceGroupDetail;
import cn.ezhizao.project.business.resourceGroup.domain.BizProductionResourceGroupSubPlan;
import cn.ezhizao.project.business.resourceGroup.service.IBizProductionResourceGroupDetailService;
import cn.ezhizao.project.business.resourceGroup.service.IBizProductionResourceGroupService;
import cn.ezhizao.project.business.resourceGroup.service.IBizProductionResourceGroupSubPlanService;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 生产计划明细Controller
 *
 * @author ruoyi
 * @date 2023-11-15
 */
@RestController
@RequestMapping("/business/productionPlanDetail")
public class BizProductionPlanDetailController extends BaseController {
    @Resource
    private IBizProductionPlanDetailService bizProductionPlanDetailService;

    @Resource
    private IBizProductionResourceGroupSubPlanService bizProductionResourceGroupSubPlanService;

    @Resource
    private IBizProductionResourceGroupService bizProductionResourceGroupService;

    @Resource
    private IBizProductionResourceGroupDetailService bizProductionResourceGroupDetailService;

    @Resource
    private IBizDayworkService dayworkService;



    /**
     * 查询生产计划明细列表
     */
//  @PreAuthorize("@ss.hasPermi('business:productionPlanDetail:list')")
    @Transactional
    @PostMapping("/list")
    /*
    public AjaxResult list(@RequestBody BizProductionPlanDetail bizProductionPlanDetail) throws NoSuchFieldException, IllegalAccessException {
        // 根据id获取资源组明细
        List<BizProductionResourceGroupDetail> detailsList = bizProductionResourceGroupDetailService.query().eq("common_type", "person").eq("common_id", SecurityUtils.getLoginUser().getUser().getUserId()).list();
        List<Long> ids = new ArrayList<>();
        for (BizProductionResourceGroupDetail detail : detailsList) {
            ids.add(detail.getProductionResourceGroupId());
        }
        if(ids.isEmpty()){
            return null;
        }

        //获取资源组
        List<BizProductionResourceGroup> groupList = bizProductionResourceGroupService.query().in("id", ids).eq("dept_id",bizProductionPlanDetail.getDeptId()).list();

        //根据资源组获取分配的生产计划
        List<Long> groupIds = new ArrayList<>();
        for (BizProductionResourceGroup group : groupList) {
            groupIds.add(group.getId());
        }
        if (groupIds.isEmpty()){
            return null;
        }
        List<BizProductionResourceGroupSubPlan> groupSubPlans = bizProductionResourceGroupSubPlanService.query().in("resource_group_id", groupIds).eq("deleted",0).list();

        //根据分配的计划，查询计划单
        List<Long> planIds = new ArrayList<>();
        for (BizProductionResourceGroupSubPlan groupSubPlan : groupSubPlans) {
            planIds.add(groupSubPlan.getProductionPlanDetailId());
        }
        if (planIds.isEmpty()){
            return null;
        }
//        QueryWrapper<BizProductionPlanDetail> queryWrapper = new QueryWrapper<>();
//        queryWrapper.in("id", planIds);
//        if(bizProductionPlanDetail.getKeywords()!=null) {
//            queryWrapper.and(item -> item
//                    .like("product_description", bizProductionPlanDetail.getKeywords())
//                    .or()
//                    .like("production_plan_no", bizProductionPlanDetail.getKeywords()));
//        }
        bizProductionPlanDetail.setPlanIds(planIds);
        List<BizProductionPlanDetail> list = bizProductionPlanDetailService.getInfo(bizProductionPlanDetail);
        for (BizProductionPlanDetail item: list) {
            BizDaywork daywork = new BizDaywork();
            daywork.setProductionPlanDetailId(item.getId());
            item.setDayWorkList(dayworkService.getList(daywork));
            for (BizProductionResourceGroupSubPlan subPlan:groupSubPlans) {
                if (subPlan.getProductionPlanDetailId().equals(item.getId())){
                    item.setResourceGroupId(subPlan.getResourceGroupId());
                }
            }
        }
        return success(list);
    }
    */

    public AjaxResult list(@RequestBody BizProductionPlanDetail bizProductionPlanDetail) throws NoSuchFieldException, IllegalAccessException {
        // 根据当前登录者的id，以及部门的id，获取资源组明细
        Long currentUserId = SecurityUtils.getLoginUser().getUser().getUserId();
        List<BizProductionResourceGroupDetail> detailsList = bizProductionResourceGroupDetailService
            .query()
            .eq("common_type", "person")
            .eq("common_id", currentUserId)
            .eq("dept_id",bizProductionPlanDetail.getDeptId())
            .list();
        List<BizProductionResourceGroup> bizProductionResourceGroups = bizProductionResourceGroupService.query().in("id", detailsList.isEmpty() ? Collections.singletonList(0L) : detailsList.stream().map(BizProductionResourceGroupDetail::getProductionResourceGroupId).collect(Collectors.toList())).list();
        detailsList = detailsList.stream().filter(v -> bizProductionResourceGroups.stream().anyMatch(e -> e.getId().equals(v.getProductionResourceGroupId()))).collect(Collectors.toList());
        /*
         * 将集合中的productionResourceGroupId取出来，存入一个List集合resourceGroupIds中
         * 后面会根据resourceGroupIds，获得到资源组分配的生产计划（生产计划与资源组的中间表）的集合
         */
        List<Long> resourceGroupIds = detailsList.stream()
            .map(BizProductionResourceGroupDetail::getProductionResourceGroupId)
            .collect(Collectors.toList());
        if(resourceGroupIds.isEmpty()){
            return null;
        }

        // 根据资源组id集合，获取到所有的资源组分配的生产计划（生产计划与资源组的中间表）的集合
        List<BizProductionResourceGroupSubPlan> groupSubPlans = bizProductionResourceGroupSubPlanService.query()
            .in("resource_group_id", resourceGroupIds)
            .eq("deleted",0)
            .list();

        /*
         * 将上面的中间表的集合中的生产计划id取出来，存入一个List集合planDetailIds中
         * 后面会根据planDetailIds，
         */
        List<Long> planDetailIds = groupSubPlans.stream()
            .map(BizProductionResourceGroupSubPlan::getProductionPlanDetailId)
            .collect(Collectors.toList());
        if (planDetailIds.isEmpty()){
            return null;
        }

        QueryWrapper<BizProductionPlanDetail> productionPlanDetailQueryWrapper = new QueryWrapper<>();
        String keywords = bizProductionPlanDetail.getKeywords();
        productionPlanDetailQueryWrapper.in("id", planDetailIds);
        if (!Objects.equals(keywords, "")) {
            productionPlanDetailQueryWrapper.and(item ->
                item.like("product_description", keywords)
                    .or()
                    .like("production_plan_no", keywords)
            );
        }
        List<BizProductionPlanDetail> list = bizProductionPlanDetailService.list(productionPlanDetailQueryWrapper);
        // 将获取到的ProductionPlanDetail集合的id提取到集合中
        List<Long> tempPlanDetailIds = list.stream()
                .map(BizProductionPlanDetail::getId)
                .collect(Collectors.toList());

        List<BizDaywork> allDayworks = dayworkService.query()
                .in("production_plan_detail_id", tempPlanDetailIds.isEmpty()? Collections.singletonList(0L) :  list.stream()
                        .map(BizProductionPlanDetail::getId)
                        .collect(Collectors.toList()))
                .eq("deleted",0)
                .list();
        for (BizProductionPlanDetail item: list) {
            List<BizDaywork> dayworks = allDayworks.stream()
                    .filter(d -> d.getProductionPlanDetailId().equals(item.getId()))
                    .collect(Collectors.toList());
            item.setDayWorkList(dayworks);
        }

        return success(list);
    }

    /**
     * 获取生产计划明细详细信息
     */
//  @PreAuthorize("@ss.hasPermi('business:productionPlanDetail:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id) {
        return success(bizProductionPlanDetailService.getById(id));
    }

    /**
     * 新增生产计划明细
     */
//  @PreAuthorize("@ss.hasPermi('business:productionPlanDetail:add')")
    @Log(title = "生产计划明细", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody BizProductionPlanDetail bizProductionPlanDetail) throws NoSuchFieldException, IllegalAccessException {
        setTenantId(bizProductionPlanDetail);
        return toAjax(bizProductionPlanDetailService.save(bizProductionPlanDetail));
    }

    /**
     * 修改生产计划明细
     */
//  @PreAuthorize("@ss.hasPermi('business:productionPlanDetail:edit')")
    @Log(title = "生产计划明细", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody BizProductionPlanDetail bizProductionPlanDetail) throws NoSuchFieldException, IllegalAccessException {
        setTenantId(bizProductionPlanDetail);
        return toAjax(bizProductionPlanDetailService.updateById(bizProductionPlanDetail));
    }

    /**
     * 删除生产计划明细
     */
//  @PreAuthorize("@ss.hasPermi('business:productionPlanDetail:remove')")
    @Log(title = "生产计划明细", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable List<Long> ids) {
        return toAjax(bizProductionPlanDetailService.removeBatchByIds(ids));
    }
}
