package cn.ezhizao.project.business.product.controller;

import cn.ezhizao.common.utils.poi.ExcelUtil;
import cn.ezhizao.framework.aspectj.lang.annotation.Log;
import cn.ezhizao.framework.aspectj.lang.enums.BusinessType;
import cn.ezhizao.framework.web.controller.BaseController;
import cn.ezhizao.framework.web.domain.AjaxResult;
import cn.ezhizao.framework.web.page.TableDataInfo;
import cn.ezhizao.project.business.product.domain.BizProductionPlanDetailSubDetailEquipmentUser;
import cn.ezhizao.project.business.product.service.IBizProductionPlanDetailSubDetailEquipmentUserService;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.util.List;
/**
 * 子计划对应的工序指定的使用设备Controller
 *
 * @author ezhizao
 * @date 2023-11-20
 */
@RestController
@RequestMapping("/business/equipmentUser")
public class BizProductionPlanDetailSubDetailEquipmentUserController extends BaseController
{
    @Resource
    private IBizProductionPlanDetailSubDetailEquipmentUserService bizProductionPlanDetailSubDetailEquipmentUserService;

    /**
     * 查询子计划对应的工序指定的使用设备列表
     */
//    @PreAuthorize("@ss.hasPermi('business:equipmentUser:list')")
    @GetMapping("/list")
    public TableDataInfo list(BizProductionPlanDetailSubDetailEquipmentUser bizProductionPlanDetailSubDetailEquipmentUser) throws NoSuchFieldException, IllegalAccessException
    {
        setTenantId(bizProductionPlanDetailSubDetailEquipmentUser);
//        startPage();
        List<BizProductionPlanDetailSubDetailEquipmentUser> list = bizProductionPlanDetailSubDetailEquipmentUserService.getList(bizProductionPlanDetailSubDetailEquipmentUser);
        return getDataTable(list);
    }

    /**
     * 导出子计划对应的工序指定的使用设备列表
     */
//    @PreAuthorize("@ss.hasPermi('business:equipmentUser:export')")
    @Log(title = "子计划对应的工序指定的使用设备", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, BizProductionPlanDetailSubDetailEquipmentUser bizProductionPlanDetailSubDetailEquipmentUser) throws NoSuchFieldException, IllegalAccessException
    {
        setTenantId(bizProductionPlanDetailSubDetailEquipmentUser);
        List<BizProductionPlanDetailSubDetailEquipmentUser> list = bizProductionPlanDetailSubDetailEquipmentUserService.getList(bizProductionPlanDetailSubDetailEquipmentUser);
        ExcelUtil<BizProductionPlanDetailSubDetailEquipmentUser> util = new ExcelUtil<BizProductionPlanDetailSubDetailEquipmentUser>(BizProductionPlanDetailSubDetailEquipmentUser.class);
        util.exportExcel(response, list, "子计划对应的工序指定的使用设备数据");
    }

    /**
     * 获取子计划对应的工序指定的使用设备详细信息
     */
//    @PreAuthorize("@ss.hasPermi('business:equipmentUser:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(bizProductionPlanDetailSubDetailEquipmentUserService.getById(id));
    }

    /**
     * 新增子计划对应的工序指定的使用设备
     */
//    @PreAuthorize("@ss.hasPermi('business:equipmentUser:add')")
    @Log(title = "子计划对应的工序指定的使用设备", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult save(@RequestBody List<BizProductionPlanDetailSubDetailEquipmentUser> bizProductionPlanDetailSubDetailEquipmentUser) throws NoSuchFieldException, IllegalAccessException
    {
        setTenantId(bizProductionPlanDetailSubDetailEquipmentUser);
        if(bizProductionPlanDetailSubDetailEquipmentUser.size()>0){
            BizProductionPlanDetailSubDetailEquipmentUser equipmentUser = new BizProductionPlanDetailSubDetailEquipmentUser();
            equipmentUser.setProductionPlanDetailSubDetailEquipmentId(bizProductionPlanDetailSubDetailEquipmentUser.get(0).getProductionPlanDetailSubDetailEquipmentId());
            equipmentUser.setEquipmentDetailId(bizProductionPlanDetailSubDetailEquipmentUser.get(0).getEquipmentDetailId());
            equipmentUser.setEquipmentDetailCode(bizProductionPlanDetailSubDetailEquipmentUser.get(0).getEquipmentDetailCode());
            List<BizProductionPlanDetailSubDetailEquipmentUser> oldList = bizProductionPlanDetailSubDetailEquipmentUserService.getList(equipmentUser);

            return toAjax(saveOrUpdateBatch(bizProductionPlanDetailSubDetailEquipmentUserService,bizProductionPlanDetailSubDetailEquipmentUser,oldList));
        }
        return error("未选择操作员");
    }

    /**
     * 修改子计划对应的工序指定的使用设备
     */
//    @PreAuthorize("@ss.hasPermi('business:equipmentUser:edit')")
    @Log(title = "子计划对应的工序指定的使用设备", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody BizProductionPlanDetailSubDetailEquipmentUser bizProductionPlanDetailSubDetailEquipmentUser) throws NoSuchFieldException, IllegalAccessException
    {
        setTenantId(bizProductionPlanDetailSubDetailEquipmentUser);
        return toAjax(bizProductionPlanDetailSubDetailEquipmentUserService.updateById(bizProductionPlanDetailSubDetailEquipmentUser));
    }

    /**
     * 删除子计划对应的工序指定的使用设备
     */
//    @PreAuthorize("@ss.hasPermi('business:equipmentUser:remove')")
    @Log(title = "子计划对应的工序指定的使用设备", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable List<Long> ids)
    {
        return toAjax(bizProductionPlanDetailSubDetailEquipmentUserService.removeBatchByIds(ids));
    }
}
