package cn.ezhizao.project.business.resourceGroup.controller;

import cn.ezhizao.common.utils.SecurityUtils;
import cn.ezhizao.common.utils.poi.ExcelUtil;
import cn.ezhizao.framework.aspectj.lang.annotation.Log;
import cn.ezhizao.framework.aspectj.lang.enums.BusinessType;
import cn.ezhizao.framework.web.controller.BaseController;
import cn.ezhizao.framework.web.domain.AjaxResult;
import cn.ezhizao.framework.web.page.TableDataInfo;
import cn.ezhizao.project.business.equipment.domain.BizEquipmentDetail;
import cn.ezhizao.project.business.equipment.domain.BizUserEquipment;
import cn.ezhizao.project.business.equipment.domain.BizUserEquipmentGroup;
import cn.ezhizao.project.business.equipment.service.IBizEquipmentDetailService;
import cn.ezhizao.project.business.equipment.service.IBizUserEquipmentGroupService;
import cn.ezhizao.project.business.product.domain.BizDayworkItem;
import cn.ezhizao.project.business.product.service.IBizDayworkItemService;
import cn.ezhizao.project.business.product.service.IBizProductionPlanDetailService;
import cn.ezhizao.project.business.resourceGroup.domain.BizProductionResourceGroupDetail;
import cn.ezhizao.project.business.resourceGroup.domain.BizProductionResourceGroupSubPlan;
import cn.ezhizao.project.business.resourceGroup.service.IBizProductionResourceGroupDetailService;
import cn.ezhizao.project.business.resourceGroup.service.IBizProductionResourceGroupSubPlanService;
import cn.ezhizao.project.system.domain.SysDept;
import cn.ezhizao.project.system.domain.SysUser;
import cn.ezhizao.project.system.service.ISysDeptService;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.springframework.web.bind.annotation.*;
import java.util.stream.Collectors;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.stream.Stream;

/**
 * 工段资源组明细Controller
 *
 * @author ezhizao
 * @date 2024-01-10
 */
@RestController
@RequestMapping("/business/detail")
public class BizProductionResourceGroupDetailController extends BaseController {
    @Resource
    private IBizProductionResourceGroupDetailService bizProductionResourceGroupDetailService;
    @Resource
    private IBizUserEquipmentGroupService bizUserEquipmentGroupService;

    @Resource
    private IBizProductionPlanDetailService bizProductionPlanDetailService;
    @Resource
    private IBizProductionResourceGroupSubPlanService bizProductionResourceGroupSubPlanService;

    @Resource
    private IBizDayworkItemService bizDayworkItemService;

    @Resource
    private ISysDeptService sysDeptService;


    @Resource
    private IBizEquipmentDetailService bizEquipmentDetailService;

    /**
     * 查询工段资源组明细列表
     */
    @GetMapping("/list")
    public TableDataInfo list(BizProductionResourceGroupDetail bizProductionResourceGroupDetail) throws NoSuchFieldException, IllegalAccessException {

        List<BizProductionResourceGroupDetail> list = bizProductionResourceGroupDetailService.getList(bizProductionResourceGroupDetail);
        return getDataTable(list);
    }

    /**
     * 查询工段资源组设备
     */
    @GetMapping("/getEquipmentByUidAndDid")
    public TableDataInfo getEquipmentByUidAndDid(Long planDetailId, Long deptId,Long lotId) {
        //这里需要更改，一个人可能在多个资源组
        //根据计划单号查出他在哪些资源组中
        SysUser user = SecurityUtils.getLoginUser().getUser();
        List<BizProductionResourceGroupSubPlan> productionPlanDetails = bizProductionResourceGroupSubPlanService.query().eq("production_plan_detail_id", planDetailId).list();
        List<Long> groupIds = new ArrayList<>();
        List<Long> equipmentIds = new ArrayList<>();
        for (BizProductionResourceGroupSubPlan item : productionPlanDetails
        ) {
            groupIds.add(item.getResourceGroupId());
        }
        //查出当前人员所在的资源组
        List<BizProductionResourceGroupDetail> userGroupList = bizProductionResourceGroupDetailService.query().in("production_resource_group_id", groupIds).eq("common_id", user.getUserId()).list();
        for (BizProductionResourceGroupDetail userItem : userGroupList
        ) {
            equipmentIds.add(userItem.getProductionResourceGroupId());
        }
        List<BizProductionResourceGroupDetail> list = bizProductionResourceGroupDetailService.query().in("production_resource_group_id", equipmentIds).eq("common_type", "equipmentDetail").inSql("common_id", "SELECT id FROM biz_equipment_detail WHERE equipment_status_code = 'NA'").eq("deleted",0).list();

        BizUserEquipmentGroup groupDetails = new BizUserEquipmentGroup();
        groupDetails.setUserId(user.getUserId());
        List<BizUserEquipmentGroup> groupList = bizUserEquipmentGroupService.getList(groupDetails);
        BizUserEquipmentGroup group = new BizUserEquipmentGroup();
        //根据设备筛掉正在使用
        List<Long> ids = new ArrayList<>();
        if (groupList.size() > 0) {
            group = groupList.get(0);
        }
        //判断当前人员是否绑定了设备，如果绑定了，那么只显示绑定的设备，如果没绑定设备，那么显示资源组设备
        SysDept dept = sysDeptService.selectDeptById(deptId);
//        if (group.getUserEquipmentList() != null && group.getUserEquipmentList().size() > 0) {
//            List<BizProductionResourceGroupDetail> itemList = new ArrayList<>();
//            for (BizUserEquipment item : group.getUserEquipmentList()) {
//                BizProductionResourceGroupDetail equipment = new BizProductionResourceGroupDetail();
//                equipment.setCommonId(item.getEquipmentDetailId());
//                equipment.setCommonCode(item.getEquipmentDetailCode());
//                ids.add(item.getEquipmentDetailId());
//                itemList.add(equipment);
//            }
//            //查询是否有正在使用的设备，如果有那么过滤掉
//            if (dept.getEquimentFlag() != 1) {
//                List<BizDayworkItem> screenList = bizDayworkItemService.query().in("equipment_detail_id", ids).lt("status", 2).list();
//                List<BizProductionResourceGroupDetail> newList = new ArrayList<>();
//                for (BizProductionResourceGroupDetail listItem : itemList) {
//                    boolean shouldRemove = false;
//                    for (BizDayworkItem item : screenList) {
//                        if (listItem.getCommonId().equals(item.getEquipmentDetailId())) {
//                            //虚拟设备可以多批次使用
//                            if(bizEquipmentDetailService.getById(item.getEquipmentDetailId()).getIsVirtualEquipment()!=1) {
//                                shouldRemove = true;
//                                break;
//                            }else {
//                                //如果是虚拟设备，则查询虚拟设备在哪些批正在使用，当前批不在正在使用的批里
//                                List<BizDayworkItem> userLotList = bizDayworkItemService.query().eq("equipment_detail_id", item.getEquipmentDetailId()).lt("status", 2).list();
//                                List<Long> distinctLotIds = userLotList.stream()
//                                        .map(BizDayworkItem::getLotId) // 获取每个对象的 lotId
//                                        .distinct() // 去重
//                                        .collect(Collectors.toList());
//                                if(distinctLotIds.contains(lotId)) {
//                                    shouldRemove = true;
//                                }
//                            }
//                        }
//                    }
//                    if (!shouldRemove) {
//                        newList.add(listItem);
//                    }
//                }
//                //去重
//                itemList = newList;
//                Set<BizProductionResourceGroupDetail> set = new HashSet<>(itemList);
//                itemList.clear();
//                itemList.addAll(set);
//                return getDataTable(itemList);
//            } else {
//                return getDataTable(itemList);
//            }
//        }
        //资源组绑定的设备id
        for (BizProductionResourceGroupDetail item : list) {
            ids.add(item.getCommonId());
        }
        //该工段设备不允许一机多批
        if (dept.getEquimentFlag() != 1) {
            if (ids.size() == 0) {
                return getDataTable(list);
            }
            //该资源组下的设备被占用
            List<BizDayworkItem> screenList = bizDayworkItemService.query().in("equipment_detail_id", ids).lt("status", 2).list();
            List<BizEquipmentDetail> equipmentDetails = new ArrayList<>();
            //查找虚拟设备
            for (Long item : ids) {
                QueryWrapper<BizEquipmentDetail> queryWrapper = new QueryWrapper<>();
                queryWrapper.eq("id", item);
                if (bizEquipmentDetailService.getOne(queryWrapper).getIsVirtualEquipment() == 1) {
                    equipmentDetails.add(bizEquipmentDetailService.getById(item));
                }
            }
            List<BizProductionResourceGroupDetail> newList = new ArrayList<>();
            for (BizProductionResourceGroupDetail item1 : list) {
                boolean shouldRemove = false;
                for (BizDayworkItem item2 : screenList) {
                    //如果资源组设备有正在使用的
                    if (item1.getCommonId().equals(item2.getEquipmentDetailId())) {
                        shouldRemove = true;
                    }
                }
                if (!shouldRemove) {
                    newList.add(item1);
                }
            }
            //把虚拟设备添加进去
            for (BizProductionResourceGroupDetail item1 : list) {
                for (BizEquipmentDetail item2 : equipmentDetails) {
                    //如果资源组设备有没有正在使用的
                    if (item1.getCommonId().equals(item2.getId())) {
                        newList.add(item1);
                    }
                }
            }
            //去重
            list = newList;
            Set<BizProductionResourceGroupDetail> set = new HashSet<>(list);
            list.clear();
            list.addAll(set);

            return getDataTable(list);
            //允许一机多批
        } else {
            //该资源组下的设备被占用
            List<BizDayworkItem> screenList = bizDayworkItemService.query().in("equipment_detail_id", ids).lt("status", 2).list();
            List<BizEquipmentDetail> equipmentDetails = new ArrayList<>();
            //查找虚拟设备
            for (Long item : ids) {
                QueryWrapper<BizEquipmentDetail> queryWrapper = new QueryWrapper<>();
                queryWrapper.eq("id", item);
                if (bizEquipmentDetailService.getOne(queryWrapper).getIsVirtualEquipment() == 1) {
                    equipmentDetails.add(bizEquipmentDetailService.getById(item));
                }
            }
            List<BizProductionResourceGroupDetail> newList = new ArrayList<>();
            //如果不是当前批次使用的，判断这台设备是否是跟当前批次使用的设备id一样(可能会因为一机多批，一台设备在多个批次使用，同时这台设备在当前lot也使用)
            //查看当前批次正在使用的设备
            List<BizDayworkItem> currentUse = bizDayworkItemService.query().eq("lot_id", lotId).lt("status", 2).list();
            //没使用的设备id
            List<Long> result = new ArrayList<>();
            for (BizProductionResourceGroupDetail item1 : list) {
                Boolean shouldRemove = false;
                for (BizDayworkItem item2 : screenList) {
                    //查找资源组没有使用的设备
                    //正在使用的设备id
                    List<Long> collect = screenList.stream().map(BizDayworkItem::getEquipmentDetailId).collect(Collectors.toList());
                    result = ids.stream()
                            .filter(item -> !collect.contains(item)) // 过滤掉包含在 ids 中的元素
                            .collect(Collectors.toList());
                    shouldRemove = true;
                    //只看正在使用的设备
                    if (item1.getCommonId().equals(item2.getEquipmentDetailId())) {
                        //如果正在被使用，看使用的批次是否是当前批次，如果不是当前批次，判断这个设备是否在这个批次也被使用
                        if (!item2.getLotId().equals(lotId)) {
                            if (currentUse.size() > 0) {
                                //当前批次正在使用的设备id
                                List<Long> collectUseIds = currentUse.stream().map(BizDayworkItem::getEquipmentDetailId).collect(Collectors.toList());
                                if (collectUseIds.contains(item2.getEquipmentDetailId())) {
                                    shouldRemove = true;
                                    continue;
                                } else {
                                    shouldRemove = false;
                                    break;
                                }
                            } else {
                                shouldRemove = false;
                                break;

                            }
                        }
                    }
                    else{
                        continue;
                    }
                }
                    if (!shouldRemove) {
                        newList.add(item1);
                    }
                }
            //未使用的设备
            for(Long item : result) {
                for (BizProductionResourceGroupDetail item1 : list) {
                    if (item1.getCommonId().equals(item)) {
                        newList.add(item1);
                    }
                }
            }

            //把虚拟设备添加进去
            for (BizProductionResourceGroupDetail item1 : list) {
                for (BizEquipmentDetail item2 : equipmentDetails) {
                    //如果资源组设备有没有正在使用的
                    if (item1.getCommonId().equals(item2.getId())) {
                        newList.add(item1);
                    }
                }
            }
            //去重
            list = newList;
            Set<BizProductionResourceGroupDetail> set = new HashSet<>(list);
            list.clear();
            list.addAll(set);

            return getDataTable(list);
            }


        }


    /**
     * 导出工段资源组明细列表
     */
    @Log(title = "工段资源组明细", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, BizProductionResourceGroupDetail bizProductionResourceGroupDetail) throws NoSuchFieldException, IllegalAccessException {
        List<BizProductionResourceGroupDetail> list = bizProductionResourceGroupDetailService.getList(bizProductionResourceGroupDetail);
        ExcelUtil<BizProductionResourceGroupDetail> util = new ExcelUtil<BizProductionResourceGroupDetail>(BizProductionResourceGroupDetail.class);
        util.exportExcel(response, list, "工段资源组明细数据");
    }

    /**
     * 获取工段资源组明细详细信息
     */
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id) {
        return success(bizProductionResourceGroupDetailService.getById(id));
    }

    /**
     * 新增工段资源组明细
     */
    @Log(title = "工段资源组明细", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody BizProductionResourceGroupDetail bizProductionResourceGroupDetail) throws NoSuchFieldException, IllegalAccessException {
        return toAjax(bizProductionResourceGroupDetailService.save(bizProductionResourceGroupDetail));
    }

    /**
     * 修改工段资源组明细
     */
    @Log(title = "工段资源组明细", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody BizProductionResourceGroupDetail bizProductionResourceGroupDetail) throws NoSuchFieldException, IllegalAccessException {
        return toAjax(bizProductionResourceGroupDetailService.updateById(bizProductionResourceGroupDetail));
    }

    /**
     * 删除工段资源组明细
     */
    @Log(title = "工段资源组明细", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long ids) {
        return toAjax(bizProductionResourceGroupDetailService.removeById(ids));
    }

    /**
     * 邀请协作者时能选择的资源组
     */
    @GetMapping("/getResourcGroup")
    public AjaxResult getResourcGroup(BizProductionResourceGroupDetail bizProductionResourceGroupDetail) {
        //查询BizProductionResourceGroupDetail表里通过commonId来查询资源组
        QueryWrapper<BizProductionResourceGroupDetail> queryWrapper = new QueryWrapper();
        queryWrapper.eq("common_id", bizProductionResourceGroupDetail.getCommonId());
        queryWrapper.eq("deleted", 0);
        List<BizProductionResourceGroupDetail> list = bizProductionResourceGroupDetailService.list(queryWrapper);
        //查询BizProductionResourceGroupSubPlan表里通过production_plan_detail_id来查询资源组
        QueryWrapper<BizProductionResourceGroupSubPlan> query = new QueryWrapper<>();
        query.eq("deleted", 0);
        query.eq("production_plan_detail_id", bizProductionResourceGroupDetail.getProductionPlanDetailId());
        List<BizProductionResourceGroupSubPlan> details = bizProductionResourceGroupSubPlanService.list(query);
        //存放资源组的交集
        List<BizProductionResourceGroupSubPlan> newList = new ArrayList<>();
        for (int i = 0; i < list.size(); i++) {
            for (int j = 0; j < details.size(); j++) {
                if (list.get(i).getProductionResourceGroupId().equals(details.get(j).getResourceGroupId())) {
                    newList.add(details.get(j));
                }
            }
        }
        return success(newList);
    }

}
