package cn.ezhizao.project.business.resourceGroup.service.impl;


import cn.ezhizao.project.business.product.domain.BizDaywork;
import cn.ezhizao.project.business.product.domain.BizDayworkItem;
import cn.ezhizao.project.business.product.service.impl.BizDayworkItemServiceImpl;
import cn.ezhizao.project.business.resourceGroup.domain.BizProductionResourceGroupDetail;
import cn.ezhizao.project.business.resourceGroup.domain.BizProductionResourceGroupSubPlan;
import cn.ezhizao.project.business.resourceGroup.mapper.BizProductionResourceGroupDetailMapper;
import cn.ezhizao.project.business.resourceGroup.service.IBizProductionResourceGroupDetailService;
import cn.ezhizao.project.system.domain.SysDeptUser;
import cn.ezhizao.project.system.mapper.SysDeptUserMapper;
import cn.ezhizao.project.system.service.impl.SysDeptUserServiceImpl;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * 工段资源组明细Service业务层处理
 *
 * @author ezhizao
 * @date 2024-01-10
 */
@Service
public class BizProductionResourceGroupDetailServiceImpl  extends ServiceImpl<BizProductionResourceGroupDetailMapper, BizProductionResourceGroupDetail> implements IBizProductionResourceGroupDetailService {
    @Resource
    private BizProductionResourceGroupDetailMapper bizProductionResourceGroupDetailMapper;
    @Resource
    private BizDayworkItemServiceImpl bizDayworkItemService;

    @Resource
    private SysDeptUserServiceImpl sysDeptUserService;
    @Resource
    private SysDeptUserMapper sysDeptUserMapper;

    /**
     * 查询工段资源组明细列表
     *
     * @param bizProductionResourceGroupDetail 工段资源组明细
     * @return 工段资源组明细
     */
    @Override
    public List<BizProductionResourceGroupDetail> getList(BizProductionResourceGroupDetail bizProductionResourceGroupDetail) {
        return bizProductionResourceGroupDetailMapper.getList(bizProductionResourceGroupDetail);
    }

    /**
     * 物理删除
     *
     * @param bizProductionResourceGroupDetail
     * @return 删除结果
     */
    @Override
    public int physicalDelete(BizProductionResourceGroupDetail bizProductionResourceGroupDetail) {
        return bizProductionResourceGroupDetailMapper.physicalDelete(bizProductionResourceGroupDetail);
    }

    /**
     * 根据用户id和部门id查询工段资源组明细
     *
     * @param userId 用户id
     * @param deptId 部门id
     * @return 工段资源组明细
     */
    @Override
    public List<BizProductionResourceGroupDetail> getEquipmentByUidAndDid(Long userId, Long deptId) {
        return bizProductionResourceGroupDetailMapper.getEquipmentByUidAndDid(userId, deptId);
    }

    /**
     * 删除临时人员
     *
     * @param bizDaywork
     */
    @Override
    public void deleTemporanyPeople(BizDaywork bizDaywork) {
        //存放资源组明细的userId
        List<Long> ids = new ArrayList<>();
        //存放资源组明细的部门id
        List<Long> deptIds = new ArrayList<>();
        //存放资源组明细的id
        List<Long> detailIds = new ArrayList<>();
        //通过传入的dayworkId查询资源组明细的临时人员
        QueryWrapper<BizProductionResourceGroupDetail> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("protem_daywork_id", bizDaywork.getId());
        queryWrapper.eq("protem_process_id", bizDaywork.getProtemProcessId());
        queryWrapper.eq("deleted", 0);
        List<BizProductionResourceGroupDetail> list = list(queryWrapper);
        if (list.size() > 0) {
            for (BizProductionResourceGroupDetail bizProductionResourceGroupDetail : list) {
                ids.add(bizProductionResourceGroupDetail.getCommonId());
                detailIds.add(bizProductionResourceGroupDetail.getId());
                deptIds.add(bizProductionResourceGroupDetail.getDeptId());
            }
            //过滤重复的deptId
            List<Long> collect = deptIds.stream().distinct().collect(Collectors.toList());
            Long deptId = collect.get(0);


            //存放工段人员的明细
            List<SysDeptUser> info = new ArrayList<>();
            //删除工段临时人员
            QueryWrapper<SysDeptUser> query = new QueryWrapper<>();
            query.eq("dept_id", deptId);
            query.eq("deleted", 0);
            query.and(i -> i.in("user_id", ids));
            query.eq("is_temporary", bizDaywork.getId());
            List<SysDeptUser> detail = sysDeptUserService.list(query);
            info.addAll(detail);


            if (info.size() > 0) {
                info.forEach(i -> {
                    i.setDeleted(1);
                });
                sysDeptUserService.updateBatchById(info);
            }
            //删除资源组明细的临时人员(先判断是否删除工段后，再删除资源组明细临时人员),刪除资源组，通过设备id查到资源组id
//        QueryWrapper<BizDayworkItem> query = new QueryWrapper<>();
//        query.eq("deleted",0);
//        query.eq("lot_id",bizDaywork.getLotId());
//        BizDayworkItem bizDayworkItem = bizDayworkItemService.list(query).get(0);

            bizProductionResourceGroupDetailMapper.deleResourceGroupDetailPeople(detailIds);
        }
    }
}
