package cn.ezhizao.project.business.technologicalProcess.controller;

import java.util.List;
import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;

import cn.ezhizao.common.utils.poi.ExcelUtil;
import cn.ezhizao.framework.aspectj.lang.annotation.Log;
import cn.ezhizao.framework.aspectj.lang.enums.BusinessType;
import cn.ezhizao.framework.web.controller.BaseController;
import cn.ezhizao.framework.web.domain.AjaxResult;
import cn.ezhizao.framework.web.page.TableDataInfo;
import cn.ezhizao.project.business.technologicalProcessDetail.domain.BizTechnologicalProcessDetail;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import cn.ezhizao.project.business.technologicalProcess.domain.BizTechnologicalProcess;
import cn.ezhizao.project.business.technologicalProcess.service.IBizTechnologicalProcessService;

/**
 * 工艺主表Controller
 *
 * @author ezhizao
 * @date 2023-11-13
 */
@RestController
@RequestMapping("/business/technologicalProcess")
public class BizTechnologicalProcessController extends BaseController
{
    @Resource
    private IBizTechnologicalProcessService bizTechnologicalProcessService;

    /**
     * 查询工艺主表列表
     */
    @PreAuthorize("@ss.hasPermi('base:technologicalProcess:list')")
    @GetMapping("/list")
    public TableDataInfo list(BizTechnologicalProcess bizTechnologicalProcess) throws NoSuchFieldException, IllegalAccessException
    {
        setTenantId(bizTechnologicalProcess);
        startPage();
        List<BizTechnologicalProcess> list = bizTechnologicalProcessService.getList(bizTechnologicalProcess);
        return getDataTable(list);
    }

    /**
     * 导出工艺主表列表
     */
    @PreAuthorize("@ss.hasPermi('base:technologicalProcess:export')")
    @Log(title = "工艺主表", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, BizTechnologicalProcess bizTechnologicalProcess) throws NoSuchFieldException, IllegalAccessException
    {
        setTenantId(bizTechnologicalProcess);
        List<BizTechnologicalProcess> list = bizTechnologicalProcessService.getList(bizTechnologicalProcess);
        ExcelUtil<BizTechnologicalProcess> util = new ExcelUtil<BizTechnologicalProcess>(BizTechnologicalProcess.class);
        util.exportExcel(response, list, "工艺主表数据");
    }

    /**
     * 获取工艺主表详细信息
     */
    @PreAuthorize("@ss.hasPermi('base:technologicalProcess:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(bizTechnologicalProcessService.getById(id));
    }

    /**
     * 新增工艺主表
     */
    @PreAuthorize("@ss.hasPermi('base:technologicalProcess:add')")
    @Log(title = "工艺主表", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody List<BizTechnologicalProcess> bizTechnologicalProcess) throws NoSuchFieldException, IllegalAccessException
    {

        List<BizTechnologicalProcess> oldList = bizTechnologicalProcessService.query().eq("product_id", bizTechnologicalProcess.get(0).getProductId()).list();
        boolean batch = saveOrUpdateBatch(bizTechnologicalProcessService, bizTechnologicalProcess, oldList);

        return toAjax(batch);
    }

    /**
     * 修改工艺主表
     */
    @PreAuthorize("@ss.hasPermi('base:technologicalProcess:edit')")
    @Log(title = "工艺主表", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody BizTechnologicalProcess bizTechnologicalProcess) throws NoSuchFieldException, IllegalAccessException
    {
        setTenantId(bizTechnologicalProcess);
        return toAjax(bizTechnologicalProcessService.updateById(bizTechnologicalProcess));
    }

    /**
     * 删除工艺主表
     */
    @PreAuthorize("@ss.hasPermi('base:technologicalProcess:remove')")
    @Log(title = "工艺主表", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable List<Long> ids)
    {
        return toAjax(bizTechnologicalProcessService.removeBatchByIds(ids));
    }
}
