package cn.ezhizao.project.business.technologicalProcessDetail.controller;

import java.util.List;
import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;

import cn.ezhizao.common.utils.poi.ExcelUtil;
import cn.ezhizao.framework.aspectj.lang.annotation.Log;
import cn.ezhizao.framework.aspectj.lang.enums.BusinessType;
import cn.ezhizao.framework.web.controller.BaseController;
import cn.ezhizao.framework.web.domain.AjaxResult;
import cn.ezhizao.framework.web.page.TableDataInfo;
import cn.ezhizao.project.business.technologicalProcessDetail.domain.BizTechnologicalProcessDetail;
import cn.ezhizao.project.business.technologicalProcessDetail.service.IBizTechnologicalProcessDetailService;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;


/**
 * 工艺工序中间表Controller
 *
 * @author ezhizao
 * @date 2023-11-16
 */
@RestController
@RequestMapping("/base/technologicalProcessDetail")
public class BizTechnologicalProcessDetailController extends BaseController
{
    @Resource
    private IBizTechnologicalProcessDetailService bizTechnologicalProcessDetailService;

    /**
     * 查询工艺工序中间表列表
     */
    @PreAuthorize("@ss.hasPermi('base:technologicalProcessDetail:list')")
    @GetMapping("/list")
    public TableDataInfo list(BizTechnologicalProcessDetail bizTechnologicalProcessDetail) throws NoSuchFieldException, IllegalAccessException
    {
        setTenantId(bizTechnologicalProcessDetail);
        startPage();
        List<BizTechnologicalProcessDetail> list = bizTechnologicalProcessDetailService.getList(bizTechnologicalProcessDetail);
        return getDataTable(list);
    }

    /**
     * 导出工艺工序中间表列表
     */
    @PreAuthorize("@ss.hasPermi('base:technologicalProcessDetail:export')")
    @Log(title = "工艺工序中间表", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, BizTechnologicalProcessDetail bizTechnologicalProcessDetail) throws NoSuchFieldException, IllegalAccessException
    {
        setTenantId(bizTechnologicalProcessDetail);
        List<BizTechnologicalProcessDetail> list = bizTechnologicalProcessDetailService.getList(bizTechnologicalProcessDetail);
        ExcelUtil<BizTechnologicalProcessDetail> util = new ExcelUtil<BizTechnologicalProcessDetail>(BizTechnologicalProcessDetail.class);
        util.exportExcel(response, list, "工艺工序中间表数据");
    }

    /**
     * 获取工艺工序中间表详细信息
     */
    @PreAuthorize("@ss.hasPermi('base:technologicalProcessDetail:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(bizTechnologicalProcessDetailService.getById(id));
    }

    /**
     * 新增工艺工序中间表
     */
    @PreAuthorize("@ss.hasPermi('base:technologicalProcessDetail:add')")
    @Log(title = "工艺工序中间表", businessType = BusinessType.INSERT)
    @PostMapping("/save")
    public AjaxResult save(@RequestBody List<BizTechnologicalProcessDetail> bizTechnologicalProcessDetail)
    {

        List<BizTechnologicalProcessDetail> oldList = bizTechnologicalProcessDetailService.query().eq("technological_process_id", bizTechnologicalProcessDetail.get(0).getTechnologicalProcessId()).list();
        boolean batch = saveOrUpdateBatch(bizTechnologicalProcessDetailService, bizTechnologicalProcessDetail, oldList);
        return toAjax(batch);
    }

    /**
     * 修改工艺工序中间表
     */
    @PreAuthorize("@ss.hasPermi('base:technologicalProcessDetail:edit')")
    @Log(title = "工艺工序中间表", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody BizTechnologicalProcessDetail bizTechnologicalProcessDetail) throws NoSuchFieldException, IllegalAccessException
    {
        setTenantId(bizTechnologicalProcessDetail);
        return toAjax(bizTechnologicalProcessDetailService.updateById(bizTechnologicalProcessDetail));
    }

    /**
     * 删除工艺工序中间表
     */
    @PreAuthorize("@ss.hasPermi('base:technologicalProcessDetail:remove')")
    @Log(title = "工艺工序中间表", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable List<Long> ids)
    {
        return toAjax(bizTechnologicalProcessDetailService.removeBatchByIds(ids));
    }
}
