package cn.ezhizao.project.system.controller;

import cn.ezhizao.common.constant.UserConstants;
import cn.ezhizao.common.utils.StringUtils;
import cn.ezhizao.framework.aspectj.lang.annotation.Log;
import cn.ezhizao.framework.aspectj.lang.enums.BusinessType;
import cn.ezhizao.framework.web.controller.BaseController;
import cn.ezhizao.framework.web.domain.AjaxResult;
import cn.ezhizao.project.business.product.domain.BizLotTechnologicalProcess;
import cn.ezhizao.project.business.product.service.IBizLotTechnologicalProcessService;
import cn.ezhizao.project.system.domain.SysDept;
import cn.ezhizao.project.system.service.ISysDeptService;
import cn.ezhizao.project.system.service.ISysDeptUserService;
import org.apache.commons.lang3.ArrayUtils;

import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

/**
 * 部门信息
 *
 * @author ruoyi
 */
@RestController
@RequestMapping("/system/dept")
public class SysDeptController extends BaseController
{
    @Resource
    private ISysDeptService deptService;
    @Resource
    private ISysDeptUserService deptUserService;

    @Resource
    private IBizLotTechnologicalProcessService bizLotTechnologicalProcessService;


    /**
     * 获取部门列表
     */
//    @PreAuthorize("@ss.hasPermi('system:dept:list')")
    @GetMapping("/list")
    public AjaxResult list(SysDept dept) throws NoSuchFieldException, IllegalAccessException {
        List<SysDept> departments = deptService.selectDeptList(dept);
        return success(departments);
    }

    @GetMapping("/listByPlanDetailsId")
    public AjaxResult listByPlanDetailsId(SysDept dept) throws NoSuchFieldException, IllegalAccessException {
        //判断该批是否是废品回用
        List<SysDept> departments = new ArrayList<>();
        if(dept.getIsAmend() == 1 ||dept.getIsWasteRecycling() == 1) {
            BizLotTechnologicalProcess technologicalProcess = bizLotTechnologicalProcessService.query().eq("lot_id", dept.getLotId()).eq("is_stop",0).one();
            dept.setLotTechnologicalProcessId(technologicalProcess.getId());
            departments = deptService.selectDeptListByTechnologyWasteRecyclingId(dept);
        } else {
            departments = deptService.selectDeptListByPlanDetailsId(dept);
        }
        return success(departments);
    }

    @GetMapping("/getUserDeptList")
    public AjaxResult getUserDeptList(String userId) throws NoSuchFieldException, IllegalAccessException {
        List<SysDept> departments = deptService.getUserDeptList(userId);
        return success(departments);
    }

    @GetMapping("/listForEquipment")
    public AjaxResult listForEquipment(SysDept dept) {
        List<SysDept> departments = deptService.selectDeptList(dept);
        return success(departments);
    }

    /**
     * 查询部门列表（排除节点）
     */
    @PreAuthorize("@ss.hasPermi('system:dept:list')")
    @GetMapping("/list/exclude/{deptId}")
    public AjaxResult excludeChild(@PathVariable(value = "deptId", required = false) Long deptId)
    {
        List<SysDept> depts = deptService.selectDeptList(new SysDept());
        depts.removeIf(d -> d.getDeptId().intValue() == deptId || ArrayUtils.contains(StringUtils.split(d.getAncestors(), ","), deptId + ""));
        return success(depts);
    }

    /**
     * 根据部门编号获取详细信息
     */
    @PreAuthorize("@ss.hasPermi('system:dept:query')")
    @GetMapping(value = "/{deptId}")
    public AjaxResult getInfo(@PathVariable Long deptId)
    {
        deptService.checkDeptDataScope(deptId);
        return success(deptService.selectDeptById(deptId));
    }

    /**
     * 新增部门
     */
    @PreAuthorize("@ss.hasPermi('system:dept:add')")
    @Log(title = "部门管理", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@Validated @RequestBody SysDept dept) throws NoSuchFieldException, IllegalAccessException {
        setTenantId(dept);
        if (UserConstants.NOT_UNIQUE.equals(deptService.checkDeptNameUnique(dept)))
        {
            return error("新增部门'" + dept.getDeptName() + "'失败，部门名称已存在");
        }
        dept.setCreateBy(getUsername());
        return toAjax(deptService.insertDept(dept));
    }

    /**
     * 新增或修改部门
     */
    @PreAuthorize("@ss.hasPermi('system:dept:add')")
    @Log(title = "部门管理", businessType = BusinessType.INSERT)
    @PostMapping(value = "save")
    @Transactional
    public AjaxResult save(@Validated @RequestBody SysDept dept) throws NoSuchFieldException, IllegalAccessException {
        setTenantId(dept);
        Long deptId = dept.getDeptId();
        // 判断没有id的进行新增操作
        if(deptId == null){
            if (UserConstants.NOT_UNIQUE.equals(deptService.checkDeptNameUnique(dept)))
            {
                return error("新增部门'" + dept.getDeptName() + "'失败，部门名称已存在");
            }
            dept.setCreateBy(getUsername());
            return toAjax(deptService.insertDept(dept));
        }else {
            deptService.checkDeptDataScope(deptId);
            if (UserConstants.NOT_UNIQUE.equals(deptService.checkDeptNameUnique(dept))) {
                return error("修改部门'" + dept.getDeptName() + "'失败，部门名称已存在");
            } else if (dept.getParentId().equals(deptId)) {
                return error("修改部门'" + dept.getDeptName() + "'失败，上级部门不能是自己");
            } else if (StringUtils.equals(UserConstants.DEPT_DISABLE, dept.getStatus()) && deptService.selectNormalChildrenDeptById(deptId) > 0) {
                return error("该部门包含未停用的子部门！");
            }
            dept.setUpdateBy(getUsername());
            return toAjax(deptService.updateDept(dept));
        }
    }

    /**
     * 修改部门
     */
    @PreAuthorize("@ss.hasPermi('system:dept:edit')")
    @Log(title = "部门管理", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@Validated @RequestBody SysDept dept) throws NoSuchFieldException, IllegalAccessException {
        setTenantId(dept);
        Long deptId = dept.getDeptId();
        deptService.checkDeptDataScope(deptId);
        if (UserConstants.NOT_UNIQUE.equals(deptService.checkDeptNameUnique(dept)))
        {
            return error("修改部门'" + dept.getDeptName() + "'失败，部门名称已存在");
        }
        else if (dept.getParentId().equals(deptId))
        {
            return error("修改部门'" + dept.getDeptName() + "'失败，上级部门不能是自己");
        }
        else if (StringUtils.equals(UserConstants.DEPT_DISABLE, dept.getStatus()) && deptService.selectNormalChildrenDeptById(deptId) > 0)
        {
            return error("该部门包含未停用的子部门！");
        }
        dept.setUpdateBy(getUsername());
        return toAjax(deptService.updateDept(dept));
    }

    /**
     * 删除部门
     */
    @PreAuthorize("@ss.hasPermi('system:dept:remove')")
    @Log(title = "部门管理", businessType = BusinessType.DELETE)
    @DeleteMapping("/{deptId}")
    public AjaxResult remove(@PathVariable Long deptId)
    {
        if (deptService.hasChildByDeptId(deptId))
        {
            return warn("存在下级部门,不允许删除");
        }
        if (deptService.checkDeptExistUser(deptId))
        {
            return warn("部门存在用户,不允许删除");
        }
        deptService.checkDeptDataScope(deptId);
        return toAjax(deptService.deleteDeptById(deptId));
    }

    /**
     * 通过userid获取deptName
     */
    @GetMapping("/getDeptName/{userId}")
    public AjaxResult getDeptName(@PathVariable Long userId)
    {
        String deptName = deptUserService.getDeptName(userId);
        return AjaxResult.success(deptName);
    }
}
